'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var componentRegistry = require('component-registry');
var _classCallCheck = _interopDefault(require('@babel/runtime/helpers/classCallCheck'));
var _createClass = _interopDefault(require('@babel/runtime/helpers/createClass'));
var _possibleConstructorReturn = _interopDefault(require('@babel/runtime/helpers/possibleConstructorReturn'));
var _getPrototypeOf = _interopDefault(require('@babel/runtime/helpers/getPrototypeOf'));
var _inherits = _interopDefault(require('@babel/runtime/helpers/inherits'));
var _objectSpread = _interopDefault(require('@babel/runtime/helpers/objectSpread'));
var inferno = require('inferno');
var safeUtils = require('safe-utils');
var classnames = _interopDefault(require('classnames'));
var _assertThisInitialized = _interopDefault(require('@babel/runtime/helpers/assertThisInitialized'));
var _typeof = _interopDefault(require('@babel/runtime/helpers/typeof'));
var infernoExtras = require('inferno-extras');
var infernoAnimation = require('inferno-animation');
var infernoBootstrap = require('inferno-bootstrap');
var infernoPopper = require('inferno-popper');
var isomorphicSchema = require('isomorphic-schema');
var _objectWithoutProperties = _interopDefault(require('@babel/runtime/helpers/objectWithoutProperties'));
var _defineProperty = _interopDefault(require('@babel/runtime/helpers/defineProperty'));
var _toConsumableArray = _interopDefault(require('@babel/runtime/helpers/toConsumableArray'));
var toNumber = _interopDefault(require('lodash.tonumber'));
var _bs_Label = _interopDefault(require('inferno-bootstrap/lib/Form/Label'));
var FormText = _interopDefault(require('inferno-bootstrap/lib/Form/FormText'));
var FormFeedback = _interopDefault(require('inferno-bootstrap/lib/Form/FormFeedback'));

var Interface = componentRegistry.createInterfaceClass('inferno-formlib');
/*

    Form generation components

*/

var IInputFieldWidget = new Interface({
  // Render the objects schema as a form
  name: 'IInputFieldWidget'
}); // Render an object schema as a HTML form

IInputFieldWidget.prototype.Component = 'object';
var IFormRowWidget = new Interface({
  // Render the objects schema as a form
  name: 'IFormRowWidget'
}); // Render an object schema as a HTML form

IFormRowWidget.prototype.Component = 'object';
var IListRowContainerWidget = new Interface({
  // Render the objects schema as a form
  name: 'IListRowContainerWidget'
}); // Render an object schema as a HTML form

IListRowContainerWidget.prototype.Component = 'object';
var ITranslationUtil = new Interface({
  name: 'ITranslationUtil'
}); // Get a i18n translation

ITranslationUtil.prototype.message = function () {};

var IFileUploadUtil = new Interface({
  name: 'IFileUploadUtil'
}); // Upload file to blob storage

IFileUploadUtil.prototype.upload = function (file, progress, done) {}; // Delete file from blob storage (TODO: Is this the right way?)


IFileUploadUtil.prototype.delete = function (uri) {};

var IDraggableController = new Interface({
  // This utility is looked up by name. The name is passed by
  // the dropping actor
  name: 'IDraggableController'
}); // Create an object based on dragged data

IDraggableController.prototype.getObject = function (data) {}; // Check if we can drop source on target


IDraggableController.prototype.mayDrop = function (source, target) {};

var interfaces = /*#__PURE__*/Object.freeze({
    IInputFieldWidget: IInputFieldWidget,
    IFormRowWidget: IFormRowWidget,
    IListRowContainerWidget: IListRowContainerWidget,
    ITranslationUtil: ITranslationUtil,
    IFileUploadUtil: IFileUploadUtil,
    IDraggableController: IDraggableController
});

function getWidgetAdapters (field, propPath, customWidgetsDict) {
  var InputFieldAdapter, RowAdapter;

  if (customWidgetsDict.hasOwnProperty(propPath)) {
    // Check if a custom widget has been provided, in which case call it

    /*
      <FormRows schema={formSchema} validationErrors={this.state.errors} value={this.state.value} onChange={this.didUpdate}>
        <CustomWidget propPath="user.title" fieldWidget={InputWidget} rowWidget={RowWidget} />
        <CustomWidget propPath="first_name" fieldWidget={InputWidget} rowWidget={RowWidget} />
      </FormRows>
      }
    */
    // TODO: Custom widget in list field
    // TODO: Custom widget in object field
    // TODO: Put this in utility function
    if (customWidgetsDict[propPath].fieldWidget) {
      // Mock adapter instance
      InputFieldAdapter = {
        context: field,
        Component: customWidgetsDict[propPath].fieldWidget
      };
    }

    if (customWidgetsDict[propPath].rowWidget) {
      // Mock adapter instance
      RowAdapter = {
        context: field,
        Component: customWidgetsDict[propPath].rowWidget
      };
    }
  } // TODO: Support readOnly


  if (!InputFieldAdapter) InputFieldAdapter = new IInputFieldWidget(field);
  if (!RowAdapter) RowAdapter = new IFormRowWidget(field);
  return {
    InputFieldAdapter: InputFieldAdapter,
    RowAdapter: RowAdapter
  };
}

function renderFormRows(_ref) {
  var schema = _ref.schema,
      value = _ref.value,
      selectFields = _ref.selectFields,
      omitFields = _ref.omitFields,
      lang = _ref.lang,
      validationErrors = _ref.validationErrors,
      namespace = _ref.namespace,
      inputName = _ref.inputName,
      isMounted = _ref.isMounted,
      customWidgets = _ref.customWidgets,
      onInput = _ref.onInput,
      onChange = _ref.onChange;
  var renderFields = Object.keys(schema._fields);
  var dottedNamespace = Array.isArray(namespace) ? namespace.join('.') : '';

  if (Array.isArray(selectFields) && selectFields.length > 0) {
    renderFields = renderFields.filter(function (key) {
      var dottedName = dottedNamespace ? "".concat(dottedNamespace, ".").concat(key) : key;
      var tmp = selectFields.filter(function (sel) {
        return sel === dottedName || sel.startsWith(dottedName + '.');
      });
      return tmp.length > 0;
    }); // Remove root level props so they aren't counted when we pass on to next level
    // so we can select all if none are passed

    selectFields = selectFields.filter(function (key) {
      return key.indexOf('.') >= 0;
    });
  }

  if (Array.isArray(omitFields)) {
    renderFields = renderFields.filter(function (key) {
      var dottedName = dottedNamespace ? "".concat(dottedNamespace, ".").concat(key) : key;
      return omitFields.indexOf(dottedName) < 0;
    });
  }

  var widgetAdapters = renderFields.map(function (key) {
    // Call the validationConstraint methods to figure out if the field should be validated
    var shouldValidate = schema._validationConstraints.reduce(function (prev, curr) {
      return prev && curr(value, key);
    }, true);

    if (!shouldValidate) {
      // Don't render fields that shouldn't be validated
      return;
    }

    var field = schema._fields[key];
    var validationError = safeUtils.safeGet(function () {
      return validationErrors && validationErrors.fieldErrors && validationErrors.fieldErrors[key];
    });
    var myNamespace = namespace.concat([key]); // .concat returns a new array

    var dotName = myNamespace.join('.'); // Unpack the invariant errors so they can be found by field key

    var tmpInvariantErrors = safeUtils.safeGet(function () {
      return validationErrors && validationErrors.invariantErrors && validationErrors.invariantErrors.filter(function (invErr) {
        // Pattern match field name of error with current namespace to see if it is a match
        return invErr.fields.reduce(function (prev, curr) {
          return prev || curr.indexOf(dotName) === 0;
        }, false);
      });
    });

    if (Array.isArray(tmpInvariantErrors) && tmpInvariantErrors.length > 0) {
      if (!validationError) validationError = {};
      validationError.invariantErrors = tmpInvariantErrors;
    }

    var _getWidgetAdapters = getWidgetAdapters(field, myNamespace.join('.'), customWidgets),
        InputFieldAdapter = _getWidgetAdapters.InputFieldAdapter,
        RowAdapter = _getWidgetAdapters.RowAdapter;

    return {
      validationError: validationError,
      propName: key,
      RowAdapter: RowAdapter,
      InputFieldAdapter: InputFieldAdapter
    };
  }); // Remove undefined widgets

  widgetAdapters = widgetAdapters.filter(function (item) {
    return item;
  });
  var widgets = widgetAdapters.map(function (_ref2) {
    var RowAdapter = _ref2.RowAdapter,
        InputFieldAdapter = _ref2.InputFieldAdapter,
        propName = _ref2.propName,
        validationError = _ref2.validationError;
    var Row = RowAdapter.Component;
    var InputField = InputFieldAdapter.Component;
    var doesNotRenderLabel = RowAdapter.doesNotRenderLabel();
    var myNamespace = namespace.concat([propName]); // .concat returns a new array

    var newInputName = inputName && propName ? inputName + '[' + propName + ']' : inputName || propName;
    var myId = myNamespace.join('.');
    var sharedProps = {
      id: myId,
      namespace: myNamespace,
      doesNotRenderLabel: doesNotRenderLabel,
      propName: propName,
      value: value && value[propName],
      options: {
        parentValue: value,
        lang: lang
      },
      validationError: validationError,
      formIsMounted: isMounted,
      // Callbacks
      onChange: onChange,
      onInput: onInput // TODO: Key should be namespace parent.propName

    };
    return inferno.normalizeProps(inferno.createComponentVNode(2, Row, _objectSpread({
      "adapter": RowAdapter
    }, sharedProps, {
      children: inferno.normalizeProps(inferno.createComponentVNode(2, InputField, _objectSpread({
        "adapter": InputFieldAdapter,
        "inputName": newInputName,
        "customWidgets": customWidgets,
        "selectFields": selectFields,
        "omitFields": omitFields
      }, sharedProps)))
    }), myNamespace.join('.')));
  });
  return widgets;
}

var FormRows =
/*#__PURE__*/
function (_Component) {
  _inherits(FormRows, _Component);

  function FormRows() {
    _classCallCheck(this, FormRows);

    return _possibleConstructorReturn(this, _getPrototypeOf(FormRows).apply(this, arguments));
  }

  _createClass(FormRows, [{
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      // I need this to pass to rows in order to avoid animations on first render
      this.isMounted = true;
    }
  }, {
    key: "getChildContext",
    value: function getChildContext() {
      // Pass along the existing context
      var outp = {};

      for (var k in this.context) {
        outp[k] = this.context[k];
      } // And add our extra helpers


      outp['renderHelpAsHtml'] = this.props.renderHelpAsHtml || this.context.renderHelpAsHtml || false;

      if (!this.context.hasOwnProperty('rootValue')) {
        outp['rootValue'] = this.props.value;
      }

      return outp;
    }
  }, {
    key: "render",
    value: function render() {
      var customWidgetDict = {};

      if (this.props.children) {
        var children = Array.isArray(this.props.children) ? this.props.children : [this.props.children];
        children.forEach(function (widget) {
          customWidgetDict[widget.props.propPath] = {
            fieldWidget: widget.props.fieldWidget,
            rowWidget: widget.props.rowWidget
          };
        });
      } // TODO: We should remove onInput and pass all changes down the chain to onChange


      return inferno.createVNode(1, "div", classnames('InfernoFormlib-FormRows', this.props.className), renderFormRows({
        schema: this.props.schema,
        namespace: this.props.namespace || [],
        inputName: this.props.inputName,
        value: this.props.value,
        selectFields: typeof this.props.selectFields === 'string' ? this.props.selectFields.split(',').map(function (k) {
          return k.trim();
        }) : this.props.selectFields,
        omitFields: typeof this.props.omitFields === 'string' ? this.props.omitFields.split(',').map(function (k) {
          return k.trim();
        }) : this.props.omitFields,
        lang: this.props.lang,
        validationErrors: this.props.validationErrors,
        isMounted: this.isMounted,
        customWidgets: customWidgetDict,
        onInput: this.props.onInput,
        onChange: this.props.onChange
      }), 0);
    }
  }]);

  return FormRows;
}(inferno.Component);

function getElOffset(el) {
  var de = document.documentElement;
  var box = el.getBoundingClientRect();
  var top = box.top + window.pageYOffset - de.clientTop;
  var left = box.left + window.pageXOffset - de.clientLeft;
  return {
    top: top,
    left: left
  };
}
var reDot = /\./g;
function escapeIdSelector(id) {
  return id.replace(reDot, '\\.');
}
var reStartingNr = /(^\d)/;
function generateId(namespace, suffix) {
  return (namespace.join('.') + suffix).replace(reStartingNr, '_$1');
}
/**
 * Throttle calls to avoid overloading an API or other service
 * 
 * Example:
 * 
 * const __timerName__ = '_optionsTimer'
 * throttle.call(this, 300, __timerName__).then((timer) => {
 *   _yourAsyncMethod().then((res) => {
 *     // Only do stuff if this is the latest executing timer
 *     if (this[__timerName__] === timer) {
 *       // Do something
 *     }
 *   })
 * })
 * 
 * 
 * @param {*} timeout
 * @param {*} propName
 */

function throttle(timeout, propName) {
  var _this = this;

  if (this[propName]) clearTimeout(this[propName]);
  return new Promise(function (resolve, reject) {
    var tmpTimer = _this[propName] = setTimeout(function () {
      resolve(tmpTimer);
    }, timeout);
  });
}

var _getElOffset = getElOffset;

var ActionBar =
/*#__PURE__*/
function (_Component) {
  _inherits(ActionBar, _Component);

  // TODO: Make this sticky AF
  function ActionBar(props) {
    var _this;

    _classCallCheck(this, ActionBar);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(ActionBar).apply(this, arguments));
    _this.state = {
      isSticky: false,
      stickyWidth: undefined
    };
    _this.didScroll = _this.didScroll.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.pollForMovementTimer = undefined;
    _this.offsetFlowNode = undefined;
    _this.pollForMovement = _this.pollForMovement.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    return _this;
  }

  _createClass(ActionBar, [{
    key: "didScroll",
    value: function didScroll(e) {
      var newState = {};
      var flowNode = this._flowingEl; // var isSticky = _getElOffset(flowNode).top < (window.scrollY + this.props.boundary.top) && window.scrollY < this.props.boundary.bottom

      var isSticky = _getElOffset(flowNode).top + flowNode.offsetHeight > window.visualViewport.height + window.scrollY && this.props.boundary.top < window.innerHeight + window.scrollY;

      if (isSticky !== this.state.isSticky) {
        newState['isSticky'] = isSticky;
      }

      var stickyWidth = flowNode.offsetWidth;

      if (stickyWidth !== this.state.stickyWidth) {
        newState['stickyWidth'] = stickyWidth;
      }

      if (Object.keys(newState).length > 0) {
        this.setState(newState);
      }
    }
  }, {
    key: "pollForMovement",
    value: function pollForMovement() {
      if (!this._flowingEl) {
        return this.pollForMovementTimer = requestAnimationFrame(this.pollForMovement);
      }

      var flowNode = this._flowingEl;
      var offset = flowNode.offsetTop - (window.scrollY !== undefined ? window.scrollY : window.pageYOffset);

      if (this.offsetFlowNode !== offset) {
        this.offsetFlowNode = offset;
        this.didScroll();
      }

      this.pollForMovementTimer = requestAnimationFrame(this.pollForMovement);
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      this.pollForMovementTimer = requestAnimationFrame(this.pollForMovement);
      this.didScroll();
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      if (this.pollForMovementTimer) {
        cancelAnimationFrame(this.pollForMovementTimer);
      }
    }
  }, {
    key: "render",
    value: function render(_ref) {
      var _this2 = this;

      var boundary = _ref.boundary,
          children = _ref.children;
      return inferno.createVNode(1, "div", classnames("InfernoFormlib-ActionBarContainer", this.props.className), [inferno.createVNode(1, "div", classnames("InfernoFormlib-ActionBar", {
        "InfernoFormlib-ActionBar--hidden": this.state.isSticky
      }), children, 0, null, null, function (e) {
        return _this2._flowingEl = e;
      }), inferno.createVNode(1, "div", classnames("InfernoFormlib-ActionBar InfernoFormlib-StickyActionBar", {
        "InfernoFormlib-StickyActionBar--hidden": !this.state.isSticky
      }), children, 0, {
        "style": {
          bottom: boundary.bottom,
          width: this.state.stickyWidth
        }
      })], 4);
    }
  }]);

  return ActionBar;
}(inferno.Component);

function renderString(label, lang, fallbackLabel) {
  if (_typeof(label) === 'object') {
    return label['i18n'];
  }

  var i18n = componentRegistry.globalRegistry.getUtility(ITranslationUtil, undefined, undefined);
  return i18n && label ? i18n.message(label, lang) || label : fallbackLabel || label;
}
function renderVariables(field, text) {
  if (field === undefined) {
    return text;
  }

  var outp = text;
  Object.getOwnPropertyNames(field).forEach(function (key) {
    // This should be faster than dynamic regex
    // https://jsperf.com/replace-vs-split-join-vs-replaceall/23
    var tmp = key[0] === '_' ? key.substring(1) : key;
    outp = outp.split("${".concat(tmp, "}")).join(field[key]);
  });
  return outp;
}

var AutocompleteItem =
/*#__PURE__*/
function (_Component) {
  _inherits(AutocompleteItem, _Component);

  function AutocompleteItem(props) {
    var _this;

    _classCallCheck(this, AutocompleteItem);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(AutocompleteItem).call(this, props));
    _this.didClick = _this.didClick.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    return _this;
  }

  _createClass(AutocompleteItem, [{
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      if (nextProps.isMarked && !this.props.isMarked) {
        // Check if it is visible
        var domNode = infernoExtras.findDOMNode(this);
        var currBottom = domNode.offsetTop + domNode.offsetHeight;

        if (currBottom > domNode.parentNode.offsetHeight + domNode.parentNode.scrollTop) {
          domNode.parentNode.scrollTop = currBottom - domNode.parentNode.offsetHeight;
        } else if (domNode.offsetTop < domNode.parentNode.scrollTop) {
          domNode.parentNode.scrollTop = domNode.offsetTop;
        }
      }
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      infernoAnimation.animateOnAdd(infernoExtras.findDOMNode(this), 'InfernoFormlib-AutocompleteItem--Animation');
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      infernoAnimation.animateOnRemove(infernoExtras.findDOMNode(this), 'InfernoFormlib-AutocompleteItem--Animation');
    }
  }, {
    key: "didClick",
    value: function didClick(e) {
      e.preventDefault();
      this.props.onSelect(this.props.value);
    }
  }, {
    key: "render",
    value: function render() {
      var cls = {
        'InfernoFormlib-AutocompleteItem': true,
        'InfernoFormlib-AutocompleteItem--Highlight': this.props.isMarked
      };
      return inferno.createComponentVNode(2, infernoBootstrap.DropdownItem, {
        "className": classnames(cls),
        "onClick": this.didClick,
        children: this.props.value.title
      });
    }
  }]);

  return AutocompleteItem;
}(inferno.Component);

function _getOptionsAsync(value, options, context) {
  var currentMarked = this.state.options[this.state.markAtIndex];
  return this.props.adapter.context.getOptionsAsync(value, options, context).then(function (results) {
    // Maintain selection when updating list if possible
    var markAtIndex = currentMarked && results.reduce(function (curr, next, index) {
      return curr === undefined && next.name === currentMarked.name ? index : curr;
    }, undefined);
    return {
      options: results,
      isOpen: true,
      markAtIndex: markAtIndex || 0 // Default to select top most option

    };
  }).catch(function () {
    return {
      options: [{
        name: value,
        title: 'Fel när data laddades...'
      }],
      isOpen: true,
      markAtIndex: undefined
    };
  });
}

function TypeAhead(props) {
  if (!props.markedValue) {
    return null;
  } else {
    var text = props.text;
    var suffix = props.markedValue.title.toLowerCase().indexOf(text.toLowerCase()) === 0 ? props.markedValue.title.substr(text.length) : '';
    return inferno.createVNode(1, "div", "InfernoFormlib-AutoComplete-TypeAhead", [inferno.createVNode(1, "span", "InfernoFormlib-AutoComplete-TypeAheadPrefix", text, 0), inferno.createVNode(1, "span", "InfernoFormlib-AutoComplete-TypeAheadSuffix", suffix, 0)], 4);
  }
}

var _actionKeys = {
  ArrowDown: true,
  ArrowUp: true,
  Enter: true
};

var AutoCompleteBaseWidget =
/*#__PURE__*/
function (_Component2) {
  _inherits(AutoCompleteBaseWidget, _Component2);

  function AutoCompleteBaseWidget(props) {
    var _this2;

    _classCallCheck(this, AutoCompleteBaseWidget);

    _this2 = _possibleConstructorReturn(this, _getPrototypeOf(AutoCompleteBaseWidget).call(this, props));
    var field = _this2.props.adapter.context; // Throttle API-calls (ms)

    _this2._throttleTimeout = 300;
    _this2.state = {
      text: props.value && _this2.toText(field.toFormattedString(props.value)),
      isOpen: false,
      options: [],
      markAtIndex: undefined
    };
    _this2.didGetHotKey = _this2.didGetHotKey.bind(_assertThisInitialized(_assertThisInitialized(_this2)));
    _this2.didGetInput = _this2.didGetInput.bind(_assertThisInitialized(_assertThisInitialized(_this2)));
    _this2.didGetChange = _this2.didGetChange.bind(_assertThisInitialized(_assertThisInitialized(_this2)));
    _this2.didSelect = _this2.didSelect.bind(_assertThisInitialized(_assertThisInitialized(_this2)));
    _this2.doToggle = _this2.doToggle.bind(_assertThisInitialized(_assertThisInitialized(_this2)));
    return _this2;
  }

  _createClass(AutoCompleteBaseWidget, [{
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      var field = this.props.adapter.context;
      this.setState({
        text: nextProps.value && this.toText(field.toFormattedString(nextProps.value)),
        value: nextProps.value
      });
    }
  }, {
    key: "toText",
    value: function toText(value) {
      return value && value.title;
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      document.addEventListener('keydown', this.didGetHotKey, false);
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      document.removeEventListener('keydown', this.didGetHotKey);
    }
  }, {
    key: "doToggle",
    value: function doToggle(e) {
      e && e.preventDefault();
      this.setState({
        isOpen: !this.state.isOpen
      });
    }
  }, {
    key: "didGetHotKey",
    value: function didGetHotKey(e) {
      // Use for navigation and enter only!
      console.log('DID PRESS: ' + e);

      if (_actionKeys[e.key] && this.state.markAtIndex !== undefined) {
        // So we got a hotkey
        var domEl = infernoExtras.findDOMNode(this._inputNode);

        if (domEl === e.target) {
          // And it was fired in our input box
          if (e.key === 'ArrowUp') {
            this.setState({
              markAtIndex: this.state.markAtIndex <= 0 ? this.state.options.length - 1 : this.state.markAtIndex - 1
            });
            e.preventDefault();
          } else if (e.key === 'ArrowDown') {
            this.setState({
              markAtIndex: this.state.markAtIndex >= this.state.options.length - 1 ? 0 : this.state.markAtIndex + 1
            });
            e.preventDefault();
          } else if (e.key === 'Enter') {
            if (this.state.markAtIndex !== undefined) {
              this.didSelect(this.state.options[this.state.markAtIndex]);
            }

            e.preventDefault();
          }
        }
      }
    }
  }, {
    key: "didGetInput",
    value: function didGetInput(e) {
      var _this3 = this;

      // TODO: How do we supply these because they could be used to get options
      var options = {
        search: e.target.value
      };

      if (e.target.value !== this.state.text && e.target.value.length > 1) {
        var __timerName__ = '_optionsTimer';
        throttle.call(this, this._throttleTimeout, __timerName__).then(function (timer) {
          var context = undefined;

          _getOptionsAsync.call(_this3, undefined, options, context).then(function (res) {
            // Only setState if this is the latest executing timer
            if (_this3[__timerName__] === timer) {
              _this3.setState(res);
            }
          });
        });
      }

      this.setState({
        text: e.target.value,
        markAtIndex: e.target.value ? this.state.markAtIndex : undefined
      });
    }
  }, {
    key: "didGetChange",
    value: function didGetChange(e) {// Not doing anything for now
    }
  }, {
    key: "didSelect",
    value: function didSelect(value) {
      var field = this.props.adapter.context;
      this.setState({
        options: [],
        isOpen: false,
        markAtIndex: undefined
      });
      this.props.onChange(this.props.propName, field.fromString(value)); // Blur focus to emphasize that selection has been done

      infernoExtras.findDOMNode(this._inputNode).blur();
    }
  }, {
    key: "renderInput",
    value: function renderInput(_ref) {
      var _this4 = this;

      var inputName = _ref.inputName,
          namespace = _ref.namespace,
          options = _ref.options;
      var field = this.props.adapter.context;
      var cls = {
        "form-control": true,
        "form-control-danger": this.props.validationError
      };
      return inferno.createComponentVNode(2, infernoBootstrap.Input, {
        "type": "text",
        "id": generateId(namespace, '__Field'),
        "name": inputName,
        "className": classnames(cls),
        "autocomplete": "off",
        "placeholder": renderString(field.placeholder, options && options.lang),
        "readOnly": field.readOnly,
        "value": this.state.text,
        "onChange": this.didGetChange,
        "onInput": this.didGetInput
      }, null, function (node) {
        return _this4._inputNode = node;
      });
    }
  }, {
    key: "renderTypeAhead",
    value: function renderTypeAhead() {
      if (this.state.text) {
        return inferno.createComponentVNode(2, TypeAhead, {
          "text": this.state.text,
          "markedValue": this.state.options[this.state.markAtIndex]
        });
      }
    }
  }, {
    key: "renderDropdownMenu",
    value: function renderDropdownMenu() {
      var _this5 = this;

      // Only show options if we have entered text
      var options = this.state.text ? this.state.options : [];
      return inferno.createComponentVNode(2, infernoBootstrap.DropdownMenu, {
        "className": "InfernoFormlib-AutoCompleteItemContainer",
        children: options.map(function (item, index) {
          return inferno.createComponentVNode(2, AutocompleteItem, {
            "value": item,
            "isMarked": index === _this5.state.markAtIndex,
            "onSelect": _this5.didSelect
          }, index);
        })
      });
    }
  }, {
    key: "render",
    value: function render(props) {
      return inferno.createComponentVNode(2, infernoBootstrap.Dropdown, {
        "className": "InfernoFormlib-AutoCompleteField",
        "isOpen": this.state.isOpen,
        "toggle": this.doToggle,
        children: [inferno.createComponentVNode(2, infernoPopper.Target, {
          children: [this.renderInput(props), this.renderTypeAhead()]
        }), this.renderDropdownMenu()]
      });
    }
  }]);

  return AutoCompleteBaseWidget;
}(inferno.Component);

var AnyOf =
/*#__PURE__*/
function (_Component) {
  _inherits(AnyOf, _Component);

  function AnyOf(props) {
    var _this;

    _classCallCheck(this, AnyOf);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(AnyOf).call(this, props));
    _this.state = {
      value: _this.props.value
    };
    _this.didGetInput = _this.didGetInput.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.didGetChange = _this.didGetChange.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    return _this;
  }

  _createClass(AnyOf, [{
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      this.setState({
        value: nextProps.value
      });
    }
  }, {
    key: "didGetInput",
    value: function didGetInput(e) {
      var field = this.props.adapter.context;
      var newVal = field.fromString(e.target.value);
      this.setState({
        value: newVal
      });
    }
  }, {
    key: "didGetChange",
    value: function didGetChange(e) {
      this.props.onChange(this.props.propName, this.state.value);
    }
  }, {
    key: "render",
    value: function render(_ref) {
      var inputName = _ref.inputName,
          namespace = _ref.namespace,
          options = _ref.options,
          doesNotRenderLabel = _ref.doesNotRenderLabel,
          id = _ref.id;
      var field = this.props.adapter.context;
      var isValid = this.props.validationError || this.props.invariantError ? false : undefined;
      var ariaLabels = {
        'aria-invalid': isValid !== undefined,
        'aria-labelledby': doesNotRenderLabel ? undefined : id,
        'aria-label': doesNotRenderLabel ? renderString(field.label || 'inferno-formlib--InputField', options && options.lang, 'AnyOf Field') : undefined,
        'aria-required': field._isRequired ? field._isRequired : undefined
      };
      return inferno.normalizeProps(inferno.createComponentVNode(2, infernoBootstrap.Input, _objectSpread({
        "id": generateId(namespace, '__Field'),
        "name": inputName,
        "valid": isValid,
        "placeholder": renderString(field.placeholder, options && options.lang),
        "readOnly": field.readOnly,
        "value": field.toFormattedString(this.state.value)
      }, ariaLabels, {
        "onInput": this.didGetInput,
        "onChange": this.didGetChange
      })));
    }
  }]);

  return AnyOf;
}(inferno.Component);
new componentRegistry.Adapter({
  implements: IInputFieldWidget,
  adapts: isomorphicSchema.interfaces.IAnyOf,
  Component: AnyOf
});

var CheckboxWidget =
/*#__PURE__*/
function (_Component) {
  _inherits(CheckboxWidget, _Component);

  function CheckboxWidget(props) {
    var _this;

    _classCallCheck(this, CheckboxWidget);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(CheckboxWidget).call(this, props));
    _this.state = {
      isChecked: props.value
    };
    _this.didGetChange = _this.didGetChange.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    return _this;
  }

  _createClass(CheckboxWidget, [{
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      if (nextProps.value !== this.state.isChecked) {
        this.setState({
          isChecked: nextProps.value
        });
      }
    }
  }, {
    key: "didGetChange",
    value: function didGetChange(e) {
      this.props.onChange(this.props.propName, this.state.isChecked ? false : true);
    }
  }, {
    key: "render",
    value: function render(_ref) {
      var inputName = _ref.inputName,
          namespace = _ref.namespace,
          options = _ref.options,
          doesNotRenderLabel = _ref.doesNotRenderLabel,
          id = _ref.id;
      var field = this.props.adapter.context;
      var isValid = this.props.validationError || this.props.invariantError ? false : undefined;
      var ariaLabels = {
        'aria-invalid': isValid !== undefined,
        'aria-labelledby': doesNotRenderLabel ? undefined : id,
        'aria-label': doesNotRenderLabel ? renderString(field.label || 'inferno-formlib--InputField', options && options.lang, 'Bool Field') : undefined,
        'aria-required': field._isRequired ? field._isRequired : undefined
      };
      return inferno.normalizeProps(inferno.createComponentVNode(2, infernoBootstrap.Input, _objectSpread({
        "type": "checkbox",
        "id": generateId(namespace, '__Field'),
        "name": inputName
      }, ariaLabels, {
        "valid": isValid,
        "placeholder": renderString(field.placeholder, options && options.lang),
        "readOnly": field.readOnly,
        "checked": this.state.isChecked,
        "onChange": this.didGetChange
      })));
    }
  }]);

  return CheckboxWidget;
}(inferno.Component);
new componentRegistry.Adapter({
  implements: IInputFieldWidget,
  adapts: isomorphicSchema.interfaces.IBoolField,
  Component: CheckboxWidget
});

var weekStartsOn = 1;

function isProperDate(inp) {
  return typeof inp === 'string' && inp.length === 10 && !isNaN(new Date(inp).getTime());
}

function toMs(days) {
  return days * 24 * 3600000;
}

function CalendarDay(props) {
  return inferno.createVNode(1, "td", classnames("DateFieldItem", {
    "text-muted": !props.isCurrentMonth,
    "DateFieldItem--selected": props.isSelected,
    "DateFieldItem--today": props.isToday
  }), inferno.createVNode(1, "a", null, props.dayNr, 0, {
    "href": '#'
  }), 2, {
    "onClick": function onClick(e) {
      e.preventDefault();
      props.onClick(props.value);
    }
  });
}

function getCalendar(date, locale) {
  var today = new Date();
  var firstOfMonth = new Date(date.getUTCFullYear(), date.getUTCMonth(), 1);
  var showDays = [];
  var rowNr = 0; // Get first day to show

  var tmpDay = new Date(firstOfMonth.valueOf() - toMs(firstOfMonth.getUTCDay() ? firstOfMonth.getUTCDay() - weekStartsOn : 7 - weekStartsOn));

  while (rowNr === 0 || tmpDay.getUTCMonth() === date.getUTCMonth() || tmpDay.getUTCDay() - weekStartsOn !== 0) {
    showDays.push({
      dayNr: tmpDay.getUTCDate(),
      isCurrentMonth: tmpDay.getUTCMonth() === date.getUTCMonth(),
      isToday: tmpDay.getUTCFullYear() === today.getFullYear() && tmpDay.getUTCMonth() === today.getMonth() && tmpDay.getUTCDate() === today.getDate(),
      value: tmpDay.toISOString().slice(0, 10)
    });
    tmpDay = new Date(tmpDay.valueOf() + toMs(1));

    if ((tmpDay.getUTCDate() - weekStartsOn) % 7 === 0) {
      rowNr++;
    }
  }

  var weekdaysMin = renderString('inferno-formlib--DateField-weekdays', undefined, 'sö_må_ti_on_to_fr_lö').split('_');
  var months = renderString('inferno-formlib--DateField-months', undefined, 'januari_februari_mars_april_maj_juni_juli_augusti_september_oktober_november_december').split('_');
  var cal = {
    monthName: months[date.getUTCMonth()],
    year: date.getUTCFullYear(),
    dayHeaders: weekdaysMin.map(function (item, index) {
      return weekdaysMin[(index + weekStartsOn) % 7]; // rotate weekdays by weekStartsOn
    }),
    days: showDays
  };
  return cal;
}

var Calendar =
/*#__PURE__*/
function (_Component) {
  _inherits(Calendar, _Component);

  function Calendar() {
    var _getPrototypeOf2;

    var _this;

    _classCallCheck(this, Calendar);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _possibleConstructorReturn(this, (_getPrototypeOf2 = _getPrototypeOf(Calendar)).call.apply(_getPrototypeOf2, [this].concat(args)));

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "state", {
      yearSelectOpen: false,
      showSelectYear: undefined
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "doStepYear", function (e, year) {
      e && e.preventDefault();

      _this.setState({
        showSelectYear: year
      });
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "doSelectYear", function (e, year) {
      e && e.preventDefault();
      var showMonth = _this.props.showMonth;

      _this.props.onNavigate(new Date(year, showMonth.month, 2));

      _this.doToggleYearSelect();
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "doToggleYearSelect", function (e) {
      e && e.preventDefault();

      _this.setState({
        yearSelectOpen: !_this.state.yearSelectOpen,
        showSelectYear: _this.props.showMonth.year
      });
    });

    return _this;
  }

  _createClass(Calendar, [{
    key: "renderYearSelect",
    value: function renderYearSelect() {
      var _this2 = this;

      var startYear = this.state.showSelectYear - 4;
      return inferno.createVNode(1, "ul", "DateField-YearSelect", [inferno.createVNode(1, "a", "DateField-YearSelect-minus", inferno.createVNode(1, "i", null, "-", 16), 2, {
        "onClick": function onClick(e) {
          return _this2.doStepYear(e, startYear - 5);
        }
      }), [0, 1, 2, 3, 4, 5, 6, 7, 8].map(function (i) {
        return inferno.createVNode(1, "li", "DateField-YearSelect-day", inferno.createVNode(1, "a", null, startYear + i, 0, {
          "onClick": function onClick(e) {
            return _this2.doSelectYear(e, startYear + i);
          }
        }), 2);
      }), inferno.createVNode(1, "a", "DateField-YearSelect-plus", inferno.createVNode(1, "i", null, "+", 16), 2, {
        "onClick": function onClick(e) {
          return _this2.doStepYear(e, startYear + 13);
        }
      })], 0);
    }
  }, {
    key: "render",
    value: function render(props) {
      var showMonth = props.showMonth,
          popoverProps = _objectWithoutProperties(props, ["showMonth"]);

      var tmpDate = new Date(showMonth.year, showMonth.month, 2);
      var currentMonth = new Date(tmpDate);
      var calendar = getCalendar(tmpDate);
      var rows = [];
      var days = [];
      calendar.days.forEach(function (day, i) {
        days.push(inferno.createComponentVNode(2, CalendarDay, {
          "value": day.value,
          "isSelected": day.value === props.value,
          "isCurrentMonth": day.isCurrentMonth,
          "isToday": day.isToday,
          "dayNr": day.dayNr,
          "onClick": props.onSelect
        }));
        i++;

        if (i % 7 === 0) {
          rows.push(inferno.createVNode(1, "tr", null, days, 0));
          days = [];
        } else if (i === calendar.days.length) {
          rows.push(inferno.createVNode(1, "tr", null, days, 0));
          days = [];
        }
      });
      return inferno.normalizeProps(inferno.createComponentVNode(2, infernoBootstrap.Popover, _objectSpread({}, popoverProps, {
        children: [inferno.createComponentVNode(2, infernoBootstrap.PopoverHeader, {
          children: inferno.createComponentVNode(2, infernoBootstrap.Nav, {
            "style": {
              width: "100%"
            },
            children: [inferno.createComponentVNode(2, infernoBootstrap.NavItem, {
              children: inferno.createComponentVNode(2, infernoBootstrap.NavLink, {
                "href": "<",
                "onClick": function onClick(e) {
                  e.preventDefault();
                  props.onNavigate(new Date(currentMonth.setMonth(currentMonth.getMonth() - 1)));
                },
                children: '<'
              })
            }), inferno.createComponentVNode(2, infernoBootstrap.NavItem, {
              "className": "DateFieldMonth",
              children: inferno.createComponentVNode(2, infernoBootstrap.NavDropdown, {
                "isOpen": this.state.yearSelectOpen,
                "toggle": this.doToggleYearSelect,
                children: [inferno.createComponentVNode(2, infernoBootstrap.DropdownToggle, {
                  "nav": true,
                  "caret": true,
                  children: [inferno.createVNode(1, "small", null, calendar.year, 0), ' ' + calendar.monthName]
                }), inferno.createComponentVNode(2, infernoBootstrap.DropdownMenu, {
                  children: inferno.createVNode(1, "div", null, this.renderYearSelect(), 0)
                })]
              })
            }), inferno.createComponentVNode(2, infernoBootstrap.NavItem, {
              children: inferno.createComponentVNode(2, infernoBootstrap.NavLink, {
                "href": ">",
                "onClick": function onClick(e) {
                  e.preventDefault();
                  props.onNavigate(new Date(currentMonth.setMonth(currentMonth.getMonth() + 1)));
                },
                children: '>'
              })
            })]
          })
        }), inferno.createComponentVNode(2, infernoBootstrap.PopoverBody, {
          "className": "DateFieldBody",
          children: inferno.createVNode(1, "table", null, inferno.createVNode(1, "tbody", null, [inferno.createVNode(1, "tr", "DateFieldHeaderRow", calendar.dayHeaders.map(function (day) {
            return inferno.createVNode(1, "th", "DateFieldHeaderItem", day, 0);
          }), 0), rows], 0), 2, {
            "style": {
              width: "100%"
            }
          })
        }), inferno.createVNode(1, "div", "DateFieldFooter", inferno.createVNode(1, "a", null, props.value || renderString('inferno-formlib--DateField-select_date', undefined, 'select a date'), 0, {
          "href": "#showSelected",
          "onClick": props.onShowSelected
        }), 2)]
      })));
    }
  }]);

  return Calendar;
}(inferno.Component);

var InputWidget =
/*#__PURE__*/
function (_Component2) {
  _inherits(InputWidget, _Component2);

  function InputWidget(props) {
    var _this3;

    _classCallCheck(this, InputWidget);

    _this3 = _possibleConstructorReturn(this, _getPrototypeOf(InputWidget).call(this, props)); // Show the month of the current date value

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this3)), "didClickBody", function (e) {
      if (_this3.state.popoverOpen) {
        var tmpNode = e.target;
        var doesContainTarget = false;

        while (tmpNode) {
          // NOTE: Using className to check if we clicked in the calendar. Should use better test.
          if (infernoExtras.findDOMNode(_this3._elInput) === tmpNode || tmpNode.className && tmpNode.className.indexOf('InfernoFormlib-DateFieldCalendar') >= 0) {
            doesContainTarget = true;
            break;
          }

          tmpNode = tmpNode.parentNode;
        }

        if (!doesContainTarget) {
          _this3.doHidePopover();
        }
      }
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this3)), "doUpdateValue", function (newDate) {
      _this3.setState({
        value: newDate
      });

      _this3.props.onChange(_this3.props.propName, newDate);

      _this3.doHidePopover();
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this3)), "doUpdateShowMonth", function (newDate) {
      _this3.setState({
        showMonth: {
          year: newDate.getFullYear(),
          month: newDate.getMonth()
        }
      });
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this3)), "doShowSelected", function (e) {
      e.preventDefault();

      _this3.setState({
        showMonth: {
          year: parseInt(_this3.state.value.slice(0, 4)),
          month: parseInt(_this3.state.value.slice(5, 7)) - 1
        }
      });
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this3)), "didGetInput", function (e) {
      var field = _this3.props.adapter.context;
      var value = field.fromString(e.target.value);

      _this3.setState({
        value: value // popoverOpen: this.state.popoverOpen && isProperDate(value)

      });

      if (value === undefined || isProperDate(value)) {
        _this3.props.onChange(_this3.props.propName, value);
      }
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this3)), "togglePopover", function () {
      if (_this3.state.popoverOpen) {
        _this3.doHidePopover();
      } else {
        _this3.doShowPopover;
      }
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this3)), "doShowPopover", function () {
      if (!_this3.state.popoverOpen) {
        _this3.setState({
          popoverOpen: true
        });

        document.addEventListener('click', _this3.didClickBody);
      }
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this3)), "doHidePopover", function (e) {
      if (_this3.state.popoverOpen) {
        _this3.setState({
          popoverOpen: false
        });

        document.removeEventListener('click', _this3.didClickBody);
      }
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this3)), "doUnset", function (e) {
      e.preventDefault();

      _this3.props.onChange(_this3.props.propName, undefined);
    });

    var date;

    if (isProperDate(props.value)) {
      date = new Date(props.value);
    } else {
      date = new Date();
    } // Adjust internal date so UTC equals current timezone


    date = new Date(date.valueOf() - date.getTimezoneOffset() * 60000 + 12 * 3600);
    _this3.state = {
      value: props.value,
      popoverOpen: false,
      showMonth: {
        year: date.getUTCFullYear(),
        month: date.getUTCMonth()
      }
    };
    return _this3;
  }

  _createClass(InputWidget, [{
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.doHidePopover();
    }
  }, {
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      if (nextProps.value !== this.props.value) {
        this.setState({
          value: nextProps.value // popoverOpen: this.state.popoverOpen && isProperDate(nextProps.value)

        }); // Update calendar

        var date = isProperDate(nextProps.value) ? new Date(nextProps.value) : new Date(); // Adjust internal date so UTC equals current timezone

        date = new Date(date.valueOf() - date.getTimezoneOffset() * 60000 + 12 * 3600);
        this.doUpdateShowMonth(date);
      }
    }
  }, {
    key: "render",
    value: function render(_ref) {
      var _this4 = this;

      var inputName = _ref.inputName,
          namespace = _ref.namespace,
          options = _ref.options,
          doesNotRenderLabel = _ref.doesNotRenderLabel,
          id = _ref.id;
      var field = this.props.adapter.context;
      var isValid = this.props.validationError || this.props.invariantError ? false : undefined;
      var ariaLabels = {
        'aria-invalid': isValid !== undefined,
        'aria-labelledby': doesNotRenderLabel ? undefined : id,
        'aria-label': doesNotRenderLabel ? renderString(field.label || 'inferno-formlib--DateField', options && options.lang, 'Date Field') : undefined,
        'aria-required': field._isRequired ? field._isRequired : undefined
      };
      var inputId = generateId(namespace, '__Field'); // TODO: onBlur={this.doHidePopover}

      return inferno.createComponentVNode(2, infernoPopper.Manager, {
        children: [inferno.createComponentVNode(2, infernoPopper.Target, {
          children: inferno.createComponentVNode(2, infernoBootstrap.InputGroup, {
            "className": this.props.className,
            children: [inferno.normalizeProps(inferno.createComponentVNode(2, infernoBootstrap.Input, _objectSpread({
              "className": this.props.innerClassName,
              "id": inputId,
              "name": inputName,
              "valid": isValid,
              "placeholder": renderString(field.placeholder, options && options.lang),
              "readOnly": field.readOnly,
              "value": field.toFormattedString(this.state.value)
            }, ariaLabels, {
              "onFocus": this.doShowPopover,
              "onBlur": this.didClickBody,
              "onInput": this.didGetInput
            }), null, function (el) {
              return _this4._elInput = el;
            })), this.props.children, inferno.createVNode(1, "a", "InfernoFormlib-DateField-unset", renderString('inferno-formlib--DateField-clear', undefined, 'clear'), 0, {
              "href": "#clear",
              "onClick": this.doUnset
            })]
          })
        }), inferno.createComponentVNode(2, Calendar, {
          "className": "InfernoFormlib-DateFieldCalendar",
          "showMonth": this.state.showMonth,
          "value": this.state.value,
          "placement": "bottom",
          "isOpen": this.state.popoverOpen,
          "target": escapeIdSelector(inputId),
          "onNavigate": this.doUpdateShowMonth,
          "onShowSelected": this.doShowSelected,
          "onSelect": this.doUpdateValue
        })]
      }, null, function (el) {
        return _this4._el = el;
      });
    }
  }]);

  return InputWidget;
}(inferno.Component);
new componentRegistry.Adapter({
  implements: IInputFieldWidget,
  adapts: isomorphicSchema.interfaces.IDateField,
  Component: InputWidget
});

function pad(inp) {
  if (inp === undefined) {
    return;
  } else if (inp.toString().length === 1) {
    return '0' + inp;
  } else {
    return inp;
  }
}

function isProperDateTime(inp) {
  return !isNaN(new Date(inp).getTime());
}

var InputWidget$1 =
/*#__PURE__*/
function (_Component) {
  _inherits(InputWidget$$1, _Component);

  function InputWidget$$1(props) {
    var _this;

    _classCallCheck(this, InputWidget$$1);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(InputWidget$$1).call(this, props));

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "didGetDateInput", function (propName, value) {
      if (typeof value === 'string') {
        var tmpNew = value.split('-');
        value = _this.state.value || new Date();
        value.setFullYear(tmpNew[0]);
        value.setMonth(tmpNew[1] - 1);
        value.setDate(tmpNew[2]);
      }

      _this.setState({
        value: value,
        popoverOpen: _this.state.popoverOpen && isProperDateTime(value)
      });

      if (value === undefined || isProperDateTime(value)) {
        _this.props.onChange(_this.props.propName, value);
      }
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "didGetTimeInput", function (e) {
      e.preventDefault();
      var value = e.target.value;

      if (typeof value === 'string') {
        var tmpNew = value.split(/[^\d]/).map(function (n) {
          return parseInt(n);
        }); // Handle 12 hour clock https://www.timeanddate.com/time/am-and-pm.html

        if (value.indexOf('pm') > 0) {
          if (tmpNew[0] < 12) {
            tmpNew[0] += 12;
          }
        } else if (value.indexOf('am') > 0) {
          if (tmpNew[0] === 12 && tmpNew[1] > 0) {
            tmpNew[0] = 0;
          }
        }

        value = _this.state.value || new Date();
        value.setHours(tmpNew[0] || 0);
        value.setMinutes(tmpNew[1] || 0);
        value.setSeconds(tmpNew[2] || 0);
      }

      _this.setState({
        value: value,
        popoverOpen: _this.state.popoverOpen && isProperDateTime(value)
      });

      if (value === undefined || isProperDateTime(value)) {
        _this.props.onChange(_this.props.propName, value);
      }
    });

    _this.state = {
      value: props.value
    };
    return _this;
  }

  _createClass(InputWidget$$1, [{
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      this.setState({
        value: nextProps.value
      }); // Todo: Update calendar
    }
  }, {
    key: "render",
    value: function render(_ref) {
      var inputName = _ref.inputName,
          namespace = _ref.namespace,
          options = _ref.options,
          doesNotRenderLabel = _ref.doesNotRenderLabel,
          id = _ref.id;
      var field = this.props.adapter.context;
      var isValid = this.props.validationError || this.props.invariantError ? false : undefined;
      var dateStr;

      if (this.state.value) {
        var val = this.state.value;
        dateStr = "".concat(val.getFullYear(), "-").concat(pad(val.getMonth() + 1), "-").concat(pad(val.getDate()));
      }

      var dateNamespace = namespace.slice();
      dateNamespace.push('_date');
      var dummyAdapter = {
        context: new isomorphicSchema.DateField({
          placeholder: field.placeholder,
          readOnly: field.readOnly
        })
      };
      var ariaLabels = {
        'aria-invalid': isValid !== undefined,
        'aria-labelledby': doesNotRenderLabel ? undefined : id,
        'aria-label': doesNotRenderLabel ? renderString(field.label || 'inferno-formlib--DateTimeField', options && options.lang, 'DateTime Field') : undefined,
        'aria-required': field._isRequired ? field._isRequired : undefined
      };
      return inferno.createComponentVNode(2, InputWidget, {
        "className": classnames('InfernoFormlib-DateTimeGroup', {
          'InfernoFormlib-DateTimeGroup--noValue': !this.state.value
        }),
        "innerClassName": "InfernoFormlib-DateTime_date",
        "adapter": dummyAdapter,
        "inputName": inputName + '_date',
        "namespace": dateNamespace,
        "valid": isValid,
        "value": dateStr,
        "id": id,
        "doesNotRenderLabel": doesNotRenderLabel,
        "onChange": this.didGetDateInput,
        children: inferno.normalizeProps(inferno.createComponentVNode(2, infernoBootstrap.Input, _objectSpread({
          "className": "InfernoFormlib-DateTime_time",
          "id": generateId(this.props.namespace, '_time__Field'),
          "name": this.props.inputName + '_time',
          "valid": isValid,
          "placeholder": renderString('hh:mm:ss', options && options.lang),
          "readOnly": field.readOnly,
          "value": this.state.value && this.state.value.toLocaleTimeString()
        }, ariaLabels, {
          "onChange": this.didGetTimeInput
        })))
      });
    }
  }]);

  return InputWidget$$1;
}(inferno.Component);
new componentRegistry.Adapter({
  implements: IInputFieldWidget,
  adapts: isomorphicSchema.interfaces.IDateTimeField,
  Component: InputWidget$1
});

var DynamicSelectFieldWidget =
/*#__PURE__*/
function (_Component) {
  _inherits(DynamicSelectFieldWidget, _Component);

  function DynamicSelectFieldWidget(props) {
    var _this;

    _classCallCheck(this, DynamicSelectFieldWidget);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(DynamicSelectFieldWidget).call(this, props));
    _this.didGetChange = _this.didGetChange.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    return _this;
  }

  _createClass(DynamicSelectFieldWidget, [{
    key: "didGetChange",
    value: function didGetChange(e) {
      var field = this.props.adapter.context;
      this.props.onChange(this.props.propName, field.fromString(field.valueType.fromString(e.target.value)));
    }
  }, {
    key: "render",
    value: function render(_ref) {
      var inputName = _ref.inputName,
          namespace = _ref.namespace,
          options = _ref.options,
          doesNotRenderLabel = _ref.doesNotRenderLabel,
          id = _ref.id;
      var field = this.props.adapter.context;
      var selectOptions = field.getOptions(this.props.value, Object.assign({
        rootValue: this.context.rootValue
      }, options));
      var isValid = this.props.validationError || this.props.invariantError ? false : undefined;
      var lang = options.lang;
      var ariaLabels = {
        'aria-invalid': isValid !== undefined,
        'aria-labelledby': doesNotRenderLabel ? undefined : id,
        'aria-label': doesNotRenderLabel ? renderString(field.label || 'inferno-formlib--InputField', options && options.lang, 'DynamicSelect Field') : undefined,
        'aria-required': field._isRequired ? field._isRequired : undefined
      };
      return inferno.normalizeProps(inferno.createComponentVNode(2, infernoBootstrap.Input, _objectSpread({
        "type": "select",
        "id": generateId(namespace, '__Field'),
        "name": inputName,
        "valid": isValid
      }, ariaLabels, {
        "readOnly": field.readOnly,
        "value": field.valueType.toFormattedString(field.toFormattedString(this.props.value)),
        "onChange": this.didGetChange,
        children: [field.placeholder && inferno.createVNode(1, "option", null, renderString(field.placeholder, lang, undefined), 0, {
          "value": ""
        }), selectOptions.map(function (item) {
          return inferno.createVNode(1, "option", null, renderString(item.title, lang, undefined), 0, {
            "value": item.name
          });
        })]
      })));
    }
  }]);

  return DynamicSelectFieldWidget;
}(inferno.Component);
new componentRegistry.Adapter({
  implements: IInputFieldWidget,
  adapts: isomorphicSchema.interfaces.IDynamicSelectBaseField,
  Component: DynamicSelectFieldWidget
});

var InputWidget$2 =
/*#__PURE__*/
function (_Component) {
  _inherits(InputWidget, _Component);

  function InputWidget(props) {
    var _this;

    _classCallCheck(this, InputWidget);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(InputWidget).call(this, props));
    _this.state = {
      value: props.value
    };
    _this.didGetInput = _this.didGetInput.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.didGetChange = _this.didGetChange.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    return _this;
  }

  _createClass(InputWidget, [{
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      this.setState({
        value: nextProps.value
      });
    }
  }, {
    key: "didGetInput",
    value: function didGetInput(e) {
      var field = this.props.adapter.context;
      var newVal = field.fromString(e.target.value);
      this.setState({
        value: newVal
      }); // Trigger change so FormRow can update character counter

      this.props.onChange(this.props.propName, newVal);
    }
  }, {
    key: "didGetChange",
    value: function didGetChange(e) {
      this.props.onChange(this.props.propName, this.state.value);
    }
  }, {
    key: "render",
    value: function render(_ref) {
      var inputName = _ref.inputName,
          namespace = _ref.namespace,
          options = _ref.options,
          id = _ref.id,
          doesNotRenderLabel = _ref.doesNotRenderLabel;
      var field = this.props.adapter.context;
      var isValid = this.props.validationError || this.props.invariantError ? false : undefined;
      var ariaLabels = {
        'aria-invalid': isValid !== undefined,
        'aria-labelledby': doesNotRenderLabel ? undefined : id,
        'aria-label': doesNotRenderLabel ? renderString(field.label || 'inferno-formlib--InputField', options && options.lang, 'Input Field') : undefined,
        'aria-required': field._isRequired ? field._isRequired : undefined
      };
      return inferno.normalizeProps(inferno.createComponentVNode(2, infernoBootstrap.Input, _objectSpread({
        "id": generateId(namespace, '__Field'),
        "name": inputName,
        "valid": isValid,
        "placeholder": renderString(field.placeholder, options && options.lang),
        "readOnly": field.readOnly,
        "value": field.toFormattedString(this.state.value)
      }, ariaLabels, {
        "onInput": this.didGetInput,
        "onChange": this.didGetChange
      })));
    }
  }]);

  return InputWidget;
}(inferno.Component);
new componentRegistry.Adapter({
  implements: IInputFieldWidget,
  adapts: isomorphicSchema.interfaces.ITextField,
  Component: InputWidget$2
});
new componentRegistry.Adapter({
  implements: IInputFieldWidget,
  adapts: isomorphicSchema.interfaces.IIntegerField,
  Component: InputWidget$2
});

var DecimalWidget =
/*#__PURE__*/
function (_InputWidget) {
  _inherits(DecimalWidget, _InputWidget);

  function DecimalWidget() {
    _classCallCheck(this, DecimalWidget);

    return _possibleConstructorReturn(this, _getPrototypeOf(DecimalWidget).apply(this, arguments));
  }

  _createClass(DecimalWidget, [{
    key: "didGetInput",
    value: function didGetInput(e) {
      var field = this.props.adapter.context;
      var inp = e.target.value; // Preserve content in input box if ending with decimal and not has any previous decimal

      if (typeof inp === 'string' && inp[inp.length - 1] === '.' && inp.match(/\./g).length === 1) {
        return this.setState({
          value: inp
        });
      }

      var newVal = field.fromString(e.target.value);
      this.setState({
        value: newVal
      }); // Trigger change so FormRow can update character counter

      this.props.onChange(this.props.propName, newVal);
    }
  }, {
    key: "render",
    value: function render(_ref2) {
      var inputName = _ref2.inputName,
          namespace = _ref2.namespace,
          options = _ref2.options,
          id = _ref2.id,
          doesNotRenderLabel = _ref2.doesNotRenderLabel;
      var field = this.props.adapter.context;
      var isValid = this.props.validationError || this.props.invariantError ? false : undefined;
      var value = this.state.value === undefined || this.state.value === null ? '' : field.toFormattedString(this.state.value);
      var ariaLabels = {
        'aria-invalid': isValid !== undefined,
        'aria-labelledby': doesNotRenderLabel ? undefined : id,
        'aria-label': doesNotRenderLabel ? renderString(field.label || 'inferno-formlib--InputField', options && options.lang, 'Input Field') : undefined,
        'aria-required': field._isRequired ? field._isRequired : undefined
      };
      return inferno.normalizeProps(inferno.createComponentVNode(2, infernoBootstrap.Input, _objectSpread({
        "id": generateId(namespace, '__Field'),
        "name": inputName,
        "valid": isValid,
        "placeholder": renderString(field.placeholder, options && options.lang),
        "readOnly": field.readOnly,
        "value": value
      }, ariaLabels, {
        "onInput": this.didGetInput,
        "onChange": this.didGetChange
      })));
    }
  }]);

  return DecimalWidget;
}(InputWidget$2);

new componentRegistry.Adapter({
  implements: IInputFieldWidget,
  adapts: isomorphicSchema.interfaces.IDecimalField,
  Component: DecimalWidget
});

var debug = typeof window !== 'undefined' && window.debugDraggable;

function getDraggable(node) {
  for (var target = node; !target.classList.contains('InfernoFormlib-DragContainer'); target = target.parentNode) {
    if (target.parentNode === null || target.parentNode.classList.contains('InfernoFormlib-DragContainer')) {
      return target;
    }
  }

  return undefined;
}

function getDragContainer(node) {
  var draggable = getDraggable(node);

  if (draggable) {
    return draggable.parentNode;
  } else {
    return undefined;
  }
}

function handleDragStart(e) {
  if (!e.target.classList.contains('InfernoFormlib-DragItem')) {
    return;
  }

  var draggable = getDraggable(e.target);
  this.currentDragContainer = getDragContainer(draggable);
  this.currentSource = this.currentTarget = draggable.getAttribute('data-drag-index');
  e.dataTransfer.effectAllowed = 'move';
  e.dataTransfer.dropEffect = 'move'; // This is needed for drag to work in FF

  e.dataTransfer.setData('text', e.target.getAttribute('data-drag-index'));
  draggable.classList.add('InfernoFormlib-DragItem--isDragging');
  debug && _dragDebug(draggable, e);
}
function getDraggableUtil(e) {
  var isDraggable = e.dataTransfer.types.reduce(function (prev, curr) {
    return prev || curr.startsWith('inferno-formlib/draggable') ? curr : undefined;
  }, undefined);
  if (isDraggable === undefined) return;
  var utilName = isDraggable.split('.')[1];
  if (utilName === undefined) return;
  var util = componentRegistry.globalRegistry.getUtilities(IDraggableController).reduce(function (prev, curr) {
    return prev || (curr._name.toLowerCase() === utilName ? curr : undefined);
  }, undefined);
  return util;
}
var currentMouseY;
function handleDragOver(e) {
  if (e.preventDefault) e.preventDefault();
  if (currentMouseY === e.clientY) return;
  currentMouseY = e.clientY;
  e.dataTransfer.dropEffect = 'move';
  var draggable = getDraggable(e.target);
  if (!draggable) return; // We are now outside the container (in another container though...)

  if (getDragContainer(draggable) !== this.currentDragContainer) {
    // If we aren't dropping in the same container, we can look up a
    // IDraggableController to see if we are allowed to drop the drag
    var util = getDraggableUtil(e);
    if (!util || !util.mayDrop(this.currentSource, this.currentTarget)) return;
  }

  debug && console.log("OVER: " + this.currentTarget + ":" + (_isAfter(draggable, e) ? 'after' : 'before'));

  _updateDragMarkers(draggable, e);

  debug && _dragDebug(draggable, e); // Only call this once on each animation frame and if the mouse has moved vertically

  /*    debounce('handleDragOver', () => {
          if (currentMouseY === e.clientY) return
          currentMouseY = e.clientY
          e.dataTransfer.dropEffect = 'move'
  
          const draggable = getDraggable(e.target)
      
          _updateDragMarkers(draggable, e)
  
          debug && _dragDebug(draggable, e)
      }) */
}
function handleDragEnter(e) {
  var draggable = getDraggable(e.target);
  if (draggable === undefined) return;

  if (this.currentTarget !== draggable.getAttribute('data-drag-index')) {
    this.currentTarget = draggable.getAttribute('data-drag-index');
  } // _updateDragMarkers(draggable, e)

}
function handleDragLeave(e) {
  var draggable = getDraggable(e.target);

  _clearDragMarkers(draggable, 'before', 'after');
}
function handleDragEnd(e) {
  var draggable = getDraggable(e.target);
  draggable.classList.remove('InfernoFormlib-DragItem--isDragging');
  draggable.removeAttribute('draggable');
  this.currentDragContainer = undefined;
  this.currentTarget = this.currentSource = undefined;
}
function handleDrop(e) {
  // If not called, drop event will try to rediriect in FF
  e.preventDefault();
  e.stopPropagation();
  var draggable = getDraggable(e.target); // Check if we are dropping outside drag container

  if (!draggable) return;

  _clearDragMarkers(draggable, 'before', 'after');

  var util; // We are now outside the container (in another container though...)

  if (getDragContainer(draggable) !== this.currentDragContainer) {
    // If we aren't dropping in the same container, we can look up a
    // IDraggableController to see if we are allowed to drop the drag
    util = getDraggableUtil(e);
    if (!util || !util.mayDrop(this.currentSource, this.currentTarget)) return;
  }

  if (this.currentTarget === this.currentSource) {
    // Dropping on self
    return;
  } else if (this.currentSource && this.currentTarget && this.currentTarget !== this.currentSource) {
    debug && console.log("DROP: " + this.currentTarget + ":" + (_isAfter(draggable, e) ? 'after' : 'before'));
    this.didDrop(this.currentSource, parseInt(this.currentTarget) + (_isAfter(draggable, e) ? 1 : 0));
  } else if (this.currentTarget) {
    // TODO: Lookup IDropObjectPrototypeFactory
    // const utilName = e.dataTransfer.utilName
    debug && console.log("DROP, Create: " + utilName + "=>" + this.currentTarget + ":" + (_isAfter(draggable, e) ? 'after' : 'before'));
    this.didDrop(util.getObject(), parseInt(this.currentTarget) + (_isAfter(draggable, e) ? 1 : 0));
  }
}

function _updateDragMarkers(el, e) {
  if (el && !el.classList.contains('InfernoFormlib-DragItem--isDragging')) {
    if (_isAfter(el, e)) {
      // Below middle
      _setDragMarkers(el, 'after');

      _clearDragMarkers(el, 'before');
    } else {
      // Above middle
      _setDragMarkers(el, 'before');

      _clearDragMarkers(el, 'after');
    }
  }
}

function _isAfter(el, e) {
  var boundRect = el.getBoundingClientRect();
  return e.clientY > boundRect.top + boundRect.height / 2;
}

function _setDragMarkers(el) {
  if (!el) return;
  var tmp = Array.prototype.slice.call(arguments); // Remove first argument

  tmp.shift();
  tmp.forEach(function (suffix) {
    !el.classList.contains('InfernoFormlib-DragItem--' + suffix) && el.classList.add('InfernoFormlib-DragItem--' + suffix);
  });
}

function _clearDragMarkers(el) {
  if (!el) return;
  var tmp = Array.prototype.slice.call(arguments); // Remove first argument

  tmp.shift();
  tmp.forEach(function (suffix) {
    el.classList.contains('InfernoFormlib-DragItem--' + suffix) && el.classList.remove('InfernoFormlib-DragItem--' + suffix);
  });
}

function _dragDebug(el, e) {
  // TODO: Add a marker to show where the drag line is
  var boundRect = el.getBoundingClientRect();
  var dTop = el.offsetTop;
  var dHeight = el.offsetHeight;
  var dbgEl = document.getElementById('dragDebug');

  if (!dbgEl) {
    // Add to DOM
    dbgEl = document.createElement('div');
    dbgEl.setAttribute('id', 'dragDebug');
    dbgEl.setAttribute('style', 'position: absolute; left: 0; right: 0; height: 2px; background-color: red');
    document.body.appendChild(dbgEl, document.body);
  }

  dbgEl.style.top = window.scrollY + boundRect.top + 'px';
  var mouseEl = document.getElementById('mouseDragDebug');

  if (!mouseEl) {
    // Add to DOM
    mouseEl = document.createElement('div');
    mouseEl.setAttribute('id', 'mouseDragDebug');
    mouseEl.setAttribute('style', 'position: absolute; left: 0; right: 0; height: 2px; background-color: green');
    document.body.appendChild(mouseEl, document.body);
  }

  console.log(window.scrollY + e.clientY - boundRect.height / 2);
  mouseEl.style.top = window.scrollY + e.clientY + 'px';
} // TODO: getAnimationFrame!!!

var ListRowContainerWidget =
/*#__PURE__*/
function (_Component) {
  _inherits(ListRowContainerWidget, _Component);

  function ListRowContainerWidget(props) {
    var _this;

    _classCallCheck(this, ListRowContainerWidget);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(ListRowContainerWidget).call(this, props));
    _this.doMakeDraggable = _this.doMakeDraggable.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.doMakeUndraggable = _this.doMakeUndraggable.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    return _this;
  }

  _createClass(ListRowContainerWidget, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      if (!this.props.isFirstMount) {
        infernoAnimation.animateOnAdd(infernoExtras.findDOMNode(this), 'InfernoFormlib-ListFieldRow--Animation');
      }
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      var domEl = infernoExtras.findDOMNode(this);
      infernoAnimation.animateOnRemove(domEl, 'InfernoFormlib-ListFieldRow--Animation');
    }
  }, {
    key: "doMakeDraggable",
    value: function doMakeDraggable(e) {
      infernoExtras.findDOMNode(this).setAttribute('draggable', true);
    }
  }, {
    key: "doMakeUndraggable",
    value: function doMakeUndraggable(e) {
      infernoExtras.findDOMNode(this).removeAttribute('draggable');
    }
  }, {
    key: "render",
    value: function render(_ref) {
      var _this2 = this;

      var children = _ref.children,
          className = _ref.className,
          options = _ref.options;
      return inferno.createVNode(1, "div", "InfernoFormlib-ListFieldRow" + (className ? ' ' + className : ''), [inferno.createVNode(1, "div", "InfernoFormlib-DragHandle", null, 1, {
        "onMouseDown": this.doMakeDraggable,
        "onMouseUp": this.doMakeUndraggable
      }), children, inferno.createVNode(1, "div", "InfernoFormlib-ListFieldRowDeleteBtnWrapper", inferno.createComponentVNode(2, infernoBootstrap.Button, {
        "className": "InfernoFormlib-ListFieldRowDeleteBtn",
        "color": "danger",
        "onClick": function onClick(e) {
          e.preventDefault();

          _this2.props.onDelete();
        },
        children: renderString(isomorphicSchema.i18n('inferno-formlib--ListField_remove', 'Remove'), options && options.lang, 'Remove')
      }), 2)], 0, {
        "data-drag-index": this.props['data-drag-index']
      });
    }
  }]);

  return ListRowContainerWidget;
}(inferno.Component);

new componentRegistry.Adapter({
  implements: IListRowContainerWidget,
  adapts: isomorphicSchema.interfaces.IListField,
  Component: ListRowContainerWidget
});

function renderRows(_ref2) {
  var field = _ref2.field,
      value = _ref2.value,
      lang = _ref2.lang,
      namespace = _ref2.namespace,
      inputName = _ref2.inputName,
      itemKeys = _ref2.itemKeys,
      validationErrors = _ref2.validationErrors,
      customWidgets = _ref2.customWidgets,
      onInput = _ref2.onInput,
      onChange = _ref2.onChange,
      _onDelete = _ref2.onDelete,
      isMounted = _ref2.isMounted;
  if (value === undefined) return;
  return value.map(function (item, index) {
    // Respect maxLength
    if (field.maxLength !== undefined && field.maxLength <= index) return;
    var valueType = field.valueType;
    var validationError = safeUtils.safeGet(function () {
      return validationErrors.errors[index];
    }); // Support readOnly
    // Support validation constraints    

    var myNamespaceByKey = namespace.concat([itemKeys[index].key]); // .concat returns a new array

    var myNamespace = namespace.concat([index]);
    var dotName = myNamespace.join('.'); // Unpack the invariant errors so they can be found by field key

    var tmpInvariantErrors = safeUtils.safeGet(function () {
      return validationErrors && validationErrors.invariantErrors && validationErrors.invariantErrors.filter(function (invErr) {
        // Pattern match field name of error with current namespace to see if it is a match
        return invErr.fields.reduce(function (prev, curr) {
          return prev || curr.indexOf(dotName) === 0;
        }, false);
      });
    });

    if (Array.isArray(tmpInvariantErrors) && tmpInvariantErrors.length > 0) {
      if (!validationError) validationError = {};
      validationError.invariantErrors = tmpInvariantErrors;
    }

    var _getWidgetAdapters = getWidgetAdapters(valueType, myNamespaceByKey.join('.'), customWidgets),
        InputFieldAdapter = _getWidgetAdapters.InputFieldAdapter,
        RowAdapter = _getWidgetAdapters.RowAdapter;

    var InputField = InputFieldAdapter.Component;
    var Row = RowAdapter.Component; // We need to know if this should be animated

    var justAdded = itemKeys[index].justAdded;
    itemKeys[index].justAdded = false;
    var newInputName = inputName && index ? inputName + '[' + index + ']' : inputName || index;
    var ListRowContainer = new IListRowContainerWidget(field).Component;
    var doesNotRenderLabel = RowAdapter.doesNotRenderLabel();
    var sharedProps = {
      namespace: myNamespace,
      propName: index,
      value: value && value[index],
      options: {
        parentValue: value,
        lang: lang
      },
      validationError: validationError,
      formIsMounted: isMounted && !justAdded,
      doesNotRenderLabel: doesNotRenderLabel,
      id: dotName,
      // Callbacks
      onChange: onChange,
      onInput: onInput
    };
    return inferno.createComponentVNode(2, ListRowContainer, {
      "className": "InfernoFormlib-DragItem",
      "data-drag-index": index,
      "isFirstMount": !isMounted,
      "propName": index,
      "value": value[index],
      "validationError": validationError,
      "onChange": onChange,
      "onDelete": function onDelete() {
        return _onDelete(index);
      },
      children: inferno.normalizeProps(inferno.createComponentVNode(2, Row, _objectSpread({
        "adapter": RowAdapter
      }, sharedProps, {
        children: inferno.normalizeProps(inferno.createComponentVNode(2, InputField, _objectSpread({
          "adapter": InputFieldAdapter,
          "inputName": newInputName,
          "customWidgets": customWidgets
        }, sharedProps)))
      })))
    }, myNamespaceByKey.join('.'));
  });
}

function Placeholder(props) {
  return inferno.createVNode(1, "div", "InfernoFormlib-ListFieldPlaceholderContainer", inferno.createVNode(1, "div", "InfernoFormlib-ListFieldPlaceholder", inferno.createVNode(1, "div", "InfernoFormlib-ListFieldPlaceholderText", props.text, 0), 2), 2);
}

var ListFieldWidget =
/*#__PURE__*/
function (_Component) {
  _inherits(ListFieldWidget, _Component);

  function ListFieldWidget(props) {
    var _this;

    _classCallCheck(this, ListFieldWidget);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(ListFieldWidget).call(this, props)); // Perhaps we should do a capability test and throw an error if not array like

    if (props.value && !(props.value instanceof Array)) throw new Error('Value passed to ListFieldWidget is not an array');
    _this.keysNext = 0;
    _this.keys = []; // Initialise keys for passed array if any
    // Making this test less strict to support Mobx Arrays:

    if (props.value instanceof Array) {
      for (var i = 0; i < props.value.length; i++) {
        _this.keys.push({
          key: _this.keysNext,
          justAdded: false
        });

        _this.keysNext++;
      }
    }

    _this.isMounted = false;
    _this.didInput = _this.didInput.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.didUpdate = _this.didUpdate.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.doAddRow = _this.doAddRow.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.doDeleteRow = _this.doDeleteRow.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.didDrop = _this.didDrop.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.handleDragStart = handleDragStart.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.handleDragOver = handleDragOver.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.handleDragEnter = handleDragEnter.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.handleDragLeave = handleDragLeave.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.handleDragEnd = handleDragEnd.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.handleDrop = handleDrop.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    return _this;
  }

  _createClass(ListFieldWidget, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      var domEl = infernoExtras.findDOMNode(this);
      domEl.addEventListener('dragstart', this.handleDragStart, false);
      domEl.addEventListener('dragover', this.handleDragOver, false);
      domEl.addEventListener('dragenter', this.handleDragEnter, false);
      domEl.addEventListener('dragleave', this.handleDragLeave, false);
      domEl.addEventListener('dragend', this.handleDragEnd, false);
      domEl.addEventListener('drop', this.handleDrop, false);
      this.listenersAdded = true; // Because this is a container we set isMounted here instead of getting it from parent

      this.isMounted = true;
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      var domEl = infernoExtras.findDOMNode(this);
      domEl.removeEventListener('dragstart', this.handleDragStart, false);
      domEl.removeEventListener('dragover', this.handleDragOver, false);
      domEl.removeEventListener('dragenter', this.handleDragEnter, false);
      domEl.removeEventListener('dragleave', this.handleDragLeave, false);
      domEl.removeEventListener('dragend', this.handleDragEnd, false);
      domEl.removeEventListener('drop', this.handleDrop, false);
    }
  }, {
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      // Making this test less strict to support Mobx Arrays:
      if (!(nextProps.value instanceof Array) || nextProps.value.length < this.keys.length) {
        // We got undefined or fewer values than previously, need to shorten the keys array
        this.keys.splice(safeUtils.safeGet(function () {
          return nextProps.value.length;
        }, 0));
        return;
      } // New array is larger so need to add som keys


      for (var i = this.keys.length; i < nextProps.value.length; i++) {
        this.keys.push({
          key: this.keysNext,
          justAdded: true
        });
        this.keysNext++;
      }
    }
  }, {
    key: "didInput",
    value: function didInput(propName, data) {
      var newValue = _toConsumableArray(this.props.value);

      newValue[propName] = data;
      this.props.onInput && this.props.onInput(this.props.propName, newValue);
    }
  }, {
    key: "didUpdate",
    value: function didUpdate(propName, data) {
      var newValue = _toConsumableArray(this.props.value);

      newValue[propName] = data;
      this.props.onChange(this.props.propName, newValue);
    }
  }, {
    key: "doAddRow",
    value: function doAddRow(e) {
      e.preventDefault(); // Adding this because in some cases the event will keep on calling this event handler.
      // Could be due to propagation.

      e.stopPropagation();
      var value = this.props.value || [];

      var newValue = _toConsumableArray(value);

      newValue.push(undefined);
      this.props.onChange(this.props.propName, newValue);
    }
  }, {
    key: "doDeleteRow",
    value: function doDeleteRow(index) {
      var value = this.props.value;
      var newValue = value.filter(function (item, i) {
        return i !== index;
      });
      this.props.onChange(this.props.propName, newValue);
    } // TODO: This should probably be removed

  }, {
    key: "didDrop",
    value: function didDrop(source, targetIndex) {
      var newValue = _toConsumableArray(this.props.value);

      var sourceKey;
      var sourceObj;

      if (_typeof(source) === 'object') {
        sourceObj = source;
        sourceKey = {
          key: this.keysNext,
          justAdded: true
        };
        this.keysNext++;
      } else {
        var sourceIndex = source;
        sourceObj = newValue.splice(sourceIndex, 1)[0];
        sourceKey = this.keys.splice(sourceIndex, 1)[0];

        if (sourceIndex < targetIndex) {
          targetIndex--;
        }
      }

      newValue.splice(targetIndex, 0, sourceObj);
      this.keys.splice(targetIndex, 0, sourceKey);
      this.props.onChange(this.props.propName, newValue);
    }
  }, {
    key: "renderAddButton",
    value: function renderAddButton(_ref) {
      var adapter = _ref.adapter,
          value = _ref.value,
          options = _ref.options;
      var field = adapter.context;
      var nrofItems = Array.isArray(value) && value.length || 0; // Respect maxLength

      if (field.maxLength <= nrofItems) return null;
      return inferno.createVNode(1, "div", "InfernoFormlib-ListFieldActionBar", inferno.createVNode(64, "input", "btn btn-primary", null, 1, {
        "type": "button",
        "value": renderString(isomorphicSchema.i18n('inferno-formlib--ListField_add', 'Add'), options && options.lang, 'Add'),
        "onClick": this.doAddRow
      }), 2);
    }
  }, {
    key: "render",
    value: function render(_ref2) {
      var adapter = _ref2.adapter,
          value = _ref2.value,
          namespace = _ref2.namespace,
          options = _ref2.options;
      var field = adapter.context;
      var emptyArray = !Array.isArray(value) || value.length === 0;
      return inferno.createVNode(1, "div", "InfernoFormlib-ListField InfernoFormlib-DragContainer", [emptyArray && field.placeholder && inferno.createComponentVNode(2, ListFieldRow, {
        "isFirstMount": !this.props.formIsMounted,
        children: inferno.createComponentVNode(2, Placeholder, {
          "text": renderString(field.placeholder)
        })
      }, "placeholder"), renderRows({
        lang: options.lang,
        field: field,
        value: value || [],
        namespace: namespace || [],
        inputName: this.props.inputName,
        itemKeys: this.keys,
        validationErrors: this.props.validationError,
        customWidgets: this.props.customWidgets,
        onInput: this.didInput,
        onChange: this.didUpdate,
        onDelete: this.doDeleteRow,
        isMounted: this.isMounted
      }), this.renderAddButton({
        adapter: adapter,
        value: value,
        options: options
      })], 0, {
        "id": generateId(namespace, '__Field')
      });
    }
  }]);

  return ListFieldWidget;
}(inferno.Component);
new componentRegistry.Adapter({
  implements: IInputFieldWidget,
  adapts: isomorphicSchema.interfaces.IListField,
  Component: ListFieldWidget
});

var MultiSelectFieldWidget =
/*#__PURE__*/
function (_Component) {
  _inherits(MultiSelectFieldWidget, _Component);

  function MultiSelectFieldWidget(props) {
    var _this;

    _classCallCheck(this, MultiSelectFieldWidget);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(MultiSelectFieldWidget).call(this, props));
    _this.didGetChange = _this.didGetChange.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    return _this;
  }

  _createClass(MultiSelectFieldWidget, [{
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      this.setState({
        value: nextProps.value
      });
    }
  }, {
    key: "didGetChange",
    value: function didGetChange(e) {
      var field = this.props.adapter.context;
      var selectedOptions = e.target.selectedOptions;
      var values = [];

      for (var i = 0; i < selectedOptions.length; i++) {
        values.push(field.fromString(field.valueType.fromString(selectedOptions[i].value)));
      }

      this.props.onChange(this.props.propName, values);
    }
  }, {
    key: "render",
    value: function render(_ref) {
      var inputName = _ref.inputName,
          namespace = _ref.namespace,
          options = _ref.options,
          doesNotRenderLabel = _ref.doesNotRenderLabel,
          id = _ref.id;
      var field = this.props.adapter.context;
      var isValid = this.props.validationError || this.props.invariantError ? false : undefined;
      var ariaLabels = {
        'aria-invalid': isValid !== undefined,
        'aria-labelledby': doesNotRenderLabel ? undefined : id,
        'aria-label': doesNotRenderLabel ? renderString(field.label || 'inferno-formlib--InputField', options && options.lang, 'MultiSelect Field') : undefined,
        'aria-required': field._isRequired ? field._isRequired : undefined
      };
      return inferno.normalizeProps(inferno.createComponentVNode(2, infernoBootstrap.Input, _objectSpread({
        "type": "select",
        "id": generateId(namespace, '__Field'),
        "name": inputName
      }, ariaLabels, {
        "multiple": "true",
        "readOnly": field.readOnly,
        "value": field.valueType.toFormattedString(field.toFormattedString(this.props.value)),
        "valid": isValid,
        "onChange": this.didGetChange,
        children: [field.placeholder && inferno.createVNode(1, "option", null, renderString(field.placeholder, options && options.lang), 0, {
          "value": ""
        }), Array.isArray(field.options) && field.options.map(function (item) {
          return inferno.createVNode(1, "option", null, item.title, 0, {
            "value": item.name
          });
        })]
      })));
    }
  }]);

  return MultiSelectFieldWidget;
}(inferno.Component);
new componentRegistry.Adapter({
  implements: IInputFieldWidget,
  adapts: isomorphicSchema.interfaces.IMultiSelectField,
  Component: MultiSelectFieldWidget
});

function renderRows$1(_ref) {
  var schema = _ref.schema,
      value = _ref.value,
      selectFields = _ref.selectFields,
      omitFields = _ref.omitFields,
      lang = _ref.lang,
      namespace = _ref.namespace,
      inputName = _ref.inputName,
      validationErrors = _ref.validationErrors,
      isMounted = _ref.isMounted,
      customWidgets = _ref.customWidgets,
      onInput = _ref.onInput,
      onChange = _ref.onChange;
  var renderFields = Object.keys(schema._fields);
  var dottedNamespace = Array.isArray(namespace) ? namespace.join('.') : '';

  if (Array.isArray(selectFields) && selectFields.length > 0) {
    renderFields = renderFields.filter(function (key) {
      var dottedName = dottedNamespace ? "".concat(dottedNamespace, ".").concat(key) : key;
      var tmp = selectFields.filter(function (sel) {
        return sel === dottedName || sel.startsWith(dottedName + '.');
      });
      return tmp.length > 0;
    }); // Remove root level props so they aren't counted when we pass on to next level
    // so we can select all if none are passed

    selectFields = selectFields.filter(function (key) {
      return key.indexOf('.') >= 0;
    });
  }

  if (Array.isArray(omitFields)) {
    renderFields = renderFields.filter(function (key) {
      var dottedName = dottedNamespace ? "".concat(dottedNamespace, ".").concat(key) : key;
      return omitFields.indexOf(dottedName) < 0;
    });
  }

  var widgets = renderFields.map(function (propName) {
    // Call the validationConstraint methods to figure out if the field should be validated
    var shouldValidate = schema._validationConstraints.reduce(function (prev, curr) {
      return prev && curr(value, propName);
    }, true);

    if (!shouldValidate) {
      // Don't render fields that shouldn't be validated
      return;
    }

    var field = schema._fields[propName];
    var validationError = safeUtils.safeGet(function () {
      return validationErrors.fieldErrors[propName];
    }); // Support readOnly
    // Support validation constraints

    var myNamespace = namespace.concat([propName]); // .concat returns a new array

    var dotName = myNamespace.join('.'); // Unpack the invariant errors so they can be found by field key

    var tmpInvariantErrors = safeUtils.safeGet(function () {
      return validationErrors && validationErrors.invariantErrors && validationErrors.invariantErrors.filter(function (invErr) {
        // Pattern match field name of error with current namespace to see if it is a match
        return invErr.fields.reduce(function (prev, curr) {
          return prev || curr.indexOf(dotName) === 0;
        }, false);
      });
    });

    if (Array.isArray(tmpInvariantErrors) && tmpInvariantErrors.length > 0) {
      if (!validationError) validationError = {};
      validationError.invariantErrors = tmpInvariantErrors;
    }

    var _getWidgetAdapters = getWidgetAdapters(field, myNamespace.join('.'), customWidgets),
        InputFieldAdapter = _getWidgetAdapters.InputFieldAdapter,
        RowAdapter = _getWidgetAdapters.RowAdapter;

    var Row = RowAdapter.Component;
    var InputField = InputFieldAdapter.Component;
    var newInputName = inputName && propName ? inputName + '[' + propName + ']' : inputName || propName;
    var doesNotRenderLabel = RowAdapter.doesNotRenderLabel();
    var sharedProps = {
      namespace: myNamespace,
      propName: propName,
      value: value && value[propName],
      options: {
        parentValue: value,
        lang: lang
      },
      validationError: validationError,
      formIsMounted: isMounted,
      doesNotRenderLabel: doesNotRenderLabel,
      id: dotName,
      // Callbacks
      onChange: onChange,
      onInput: onInput // TODO: Key should be namespace parent.propName?

    };
    return inferno.normalizeProps(inferno.createComponentVNode(2, Row, _objectSpread({
      "adapter": RowAdapter
    }, sharedProps, {
      children: inferno.normalizeProps(inferno.createComponentVNode(2, InputField, _objectSpread({
        "adapter": InputFieldAdapter,
        "inputName": newInputName,
        "customWidgets": customWidgets,
        "selectFields": selectFields,
        "omitFields": omitFields
      }, sharedProps)))
    }), myNamespace.join('.')));
  });
  return widgets;
}

var ObjectFieldWidget =
/*#__PURE__*/
function (_Component) {
  _inherits(ObjectFieldWidget, _Component);

  function ObjectFieldWidget(props) {
    var _this;

    _classCallCheck(this, ObjectFieldWidget);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(ObjectFieldWidget).call(this, props));
    _this.isMounted = false;
    _this.didUpdate = _this.didUpdate.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.didInput = _this.didInput.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    return _this;
  }

  _createClass(ObjectFieldWidget, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      // Because this is a container we set isMounted here instead of getting it from parent
      this.isMounted = true;
    }
  }, {
    key: "didInput",
    value: function didInput(propName, data) {
      var newValue = _objectSpread({}, this.props.value);

      newValue[propName] = data;
      this.props.onInput && this.props.onInput(this.props.propName, newValue);
    }
  }, {
    key: "didUpdate",
    value: function didUpdate(propName, data) {
      var newValue = _objectSpread({}, this.props.value);

      newValue[propName] = data;
      this.props.onChange(this.props.propName, newValue);
    }
  }, {
    key: "render",
    value: function render() {
      var field = this.props.adapter.context; // Should this really be field.interface?

      var schema = field._interface ? field._interface.schema : field._schema || field.schema;

      if (!schema) {
        return null;
      }

      return inferno.createVNode(1, "div", "InfernoFormlib-ObjectField", renderRows$1({
        schema: schema,
        namespace: this.props.namespace || [],
        inputName: this.props.inputName,
        value: this.props.value,
        selectFields: typeof this.props.selectFields === 'string' ? this.props.selectFields.split(',').map(function (k) {
          return k.trim();
        }) : this.props.selectFields,
        omitFields: typeof this.props.omitFields === 'string' ? this.props.omitFields.split(',').map(function (k) {
          return k.trim();
        }) : this.props.omitFields,
        lang: this.props.options && this.props.options.lang,
        validationErrors: this.props.validationError,
        isMounted: this.isMounted,
        customWidgets: this.props.customWidgets,
        onInput: this.didInput,
        onChange: this.didUpdate
      }), 0, {
        "id": generateId(this.props.namespace, '__Field')
      });
    }
  }]);

  return ObjectFieldWidget;
}(inferno.Component);
new componentRegistry.Adapter({
  implements: IInputFieldWidget,
  adapts: isomorphicSchema.interfaces.IObjectField,
  Component: ObjectFieldWidget
});

var PasswordWidget =
/*#__PURE__*/
function (_Component) {
  _inherits(PasswordWidget, _Component);

  function PasswordWidget(props) {
    var _this;

    _classCallCheck(this, PasswordWidget);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(PasswordWidget).call(this, props));
    _this.state = {
      value: props.value
    };
    _this.didGetInput = _this.didGetInput.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.didGetChange = _this.didGetChange.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    return _this;
  }

  _createClass(PasswordWidget, [{
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      this.setState({
        value: nextProps.value
      });
    }
  }, {
    key: "didGetInput",
    value: function didGetInput(e) {
      var field = this.props.adapter.context;
      var newVal = field.fromString(e.target.value);
      this.setState({
        value: newVal
      });
    }
  }, {
    key: "didGetChange",
    value: function didGetChange(e) {
      this.props.onChange(this.props.propName, this.state.value);
    }
  }, {
    key: "render",
    value: function render(_ref) {
      var inputName = _ref.inputName,
          namespace = _ref.namespace,
          options = _ref.options,
          doesNotRenderLabel = _ref.doesNotRenderLabel,
          id = _ref.id;
      var field = this.props.adapter.context;
      var isValid = this.props.validationError || this.props.invariantError ? false : undefined;
      var ariaLabels = {
        'aria-invalid': isValid !== undefined,
        'aria-labelledby': doesNotRenderLabel ? undefined : id,
        'aria-label': doesNotRenderLabel ? renderString(field.label || 'inferno-formlib--InputField', options && options.lang, 'Password Field') : undefined,
        'aria-required': field._isRequired ? field._isRequired : undefined
      };
      return inferno.normalizeProps(inferno.createComponentVNode(2, infernoBootstrap.Input, _objectSpread({
        "type": "password",
        "id": generateId(namespace, '__Field'),
        "name": inputName,
        "placeholder": renderString(field.placeholder, options && options.lang),
        "readOnly": field.readOnly,
        "value": this.state.value,
        "valid": isValid
      }, ariaLabels, {
        "onInput": this.didGetInput,
        "onChange": this.didGetChange
      })));
    }
  }]);

  return PasswordWidget;
}(inferno.Component);
new componentRegistry.Adapter({
  implements: IInputFieldWidget,
  adapts: isomorphicSchema.interfaces.IPasswordField,
  Component: PasswordWidget
});

function _getOptionsAsync$1(nextProps) {
  var _this = this;

  // TODO: How do we supply these because they could be used to get options
  var options = undefined;
  var context = undefined;
  this.props.adapter.context.getOptionsAsync(nextProps.value, options, context).then(function (results) {
    _this.setState({
      options: results
    });
  }).catch(function () {
    _this.setState({
      options: [{
        name: nextProps.value,
        title: 'Fel när data laddades...'
      }]
    });
  });
}

var SelectAsyncBaseWidget =
/*#__PURE__*/
function (_Component) {
  _inherits(SelectAsyncBaseWidget, _Component);

  function SelectAsyncBaseWidget(props) {
    var _this2;

    _classCallCheck(this, SelectAsyncBaseWidget);

    _this2 = _possibleConstructorReturn(this, _getPrototypeOf(SelectAsyncBaseWidget).call(this, props));
    _this2.state = {
      options: [{
        name: props.value,
        title: 'Laddar...'
      }]
    };
    _this2.didGetChange = _this2.didGetChange.bind(_assertThisInitialized(_assertThisInitialized(_this2)));
    return _this2;
  }

  _createClass(SelectAsyncBaseWidget, [{
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      // Get new options from field
      if (nextProps.adapter.context !== this.props.adapter.context) {
        _getOptionsAsync$1.call(this, nextProps);
      }
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      _getOptionsAsync$1.call(this, this.props);
    }
  }, {
    key: "didGetChange",
    value: function didGetChange(e) {
      var field = this.props.adapter.context;
      this.props.onChange(this.props.propName, field.fromString(field.valueType.fromString(e.target.value)));
    }
  }, {
    key: "render",
    value: function render(_ref) {
      var inputName = _ref.inputName,
          namespace = _ref.namespace,
          options = _ref.options;
      var field = this.props.adapter.context;
      var isValid = this.props.validationError || this.props.invariantError ? false : undefined;
      return inferno.createComponentVNode(2, infernoBootstrap.Input, {
        "type": "select",
        "id": generateId(namespace, '__Field'),
        "name": inputName,
        "aria-label": inputName || 'select',
        "aria-invalid": isValid,
        "readOnly": field.readOnly,
        "value": field.valueType.toFormattedString(field.toFormattedString(this.props.value)),
        "valid": isValid,
        "onChange": this.didGetChange,
        children: [field.placeholder && inferno.createVNode(1, "option", null, renderString(field.placeholder, options && options.lang), 0, {
          "value": ""
        }), this.state.options.map(function (item) {
          return inferno.createVNode(1, "option", null, item.title, 0, {
            "value": item.name
          });
        })]
      });
    }
  }]);

  return SelectAsyncBaseWidget;
}(inferno.Component);

var SelectFieldWidget =
/*#__PURE__*/
function (_Component) {
  _inherits(SelectFieldWidget, _Component);

  function SelectFieldWidget(props) {
    var _this;

    _classCallCheck(this, SelectFieldWidget);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(SelectFieldWidget).call(this, props));
    _this.didGetChange = _this.didGetChange.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    return _this;
  }

  _createClass(SelectFieldWidget, [{
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      this.setState({
        value: nextProps.value
      });
    }
  }, {
    key: "didGetChange",
    value: function didGetChange(e) {
      var field = this.props.adapter.context;
      this.props.onChange(this.props.propName, field.fromString(field.valueType.fromString(e.target.value)));
    }
  }, {
    key: "render",
    value: function render(_ref) {
      var inputName = _ref.inputName,
          namespace = _ref.namespace,
          options = _ref.options,
          doesNotRenderLabel = _ref.doesNotRenderLabel,
          id = _ref.id;
      var field = this.props.adapter.context;
      var isValid = this.props.validationError || this.props.invariantError ? false : undefined;
      var lang = options.lang;
      var ariaLabels = {
        'aria-invalid': isValid !== undefined,
        'aria-labelledby': doesNotRenderLabel ? undefined : id,
        'aria-label': doesNotRenderLabel ? renderString(field.label || 'inferno-formlib--InputField', options && options.lang, 'Select Field') : undefined,
        'aria-required': field._isRequired ? field._isRequired : undefined
      };
      return inferno.normalizeProps(inferno.createComponentVNode(2, infernoBootstrap.Input, _objectSpread({
        "type": "select",
        "id": generateId(namespace, '__Field'),
        "name": inputName
      }, ariaLabels, {
        "readOnly": field.readOnly && 'true',
        "value": field.valueType.toFormattedString(field.toFormattedString(this.props.value)),
        "valid": isValid,
        "onChange": this.didGetChange,
        children: [field.placeholder && inferno.createVNode(1, "option", null, renderString(field.placeholder, lang, undefined), 0, {
          "value": ""
        }), Array.isArray(field.options) && field.options.map(function (item) {
          return inferno.createVNode(1, "option", null, renderString(item.title, lang, undefined), 0, {
            "value": item.name
          });
        })]
      })));
    }
  }]);

  return SelectFieldWidget;
}(inferno.Component);
new componentRegistry.Adapter({
  implements: IInputFieldWidget,
  adapts: isomorphicSchema.interfaces.ISelectField,
  Component: SelectFieldWidget
});

var TextAreaWidget =
/*#__PURE__*/
function (_Component) {
  _inherits(TextAreaWidget, _Component);

  function TextAreaWidget(props) {
    var _this;

    _classCallCheck(this, TextAreaWidget);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(TextAreaWidget).call(this, props));
    _this.state = {
      value: props.value
    };
    _this.didGetInput = _this.didGetInput.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.didGetChange = _this.didGetChange.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    return _this;
  }

  _createClass(TextAreaWidget, [{
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      this.setState({
        value: nextProps.value
      });
    }
  }, {
    key: "didGetInput",
    value: function didGetInput(e) {
      var field = this.props.adapter.context;
      var newVal = field.fromString(e.target.value);
      this.setState({
        value: newVal
      }); // Trigger change so FormRow can update character counter

      this.props.onChange(this.props.propName, newVal);
    }
  }, {
    key: "didGetChange",
    value: function didGetChange(e) {
      this.props.onChange(this.props.propName, this.state.value);
    }
  }, {
    key: "render",
    value: function render(_ref) {
      var inputName = _ref.inputName,
          namespace = _ref.namespace,
          options = _ref.options,
          doesNotRenderLabel = _ref.doesNotRenderLabel,
          id = _ref.id;
      var field = this.props.adapter.context;
      var isValid = this.props.validationError || this.props.invariantError ? false : undefined;
      var ariaLabels = {
        'aria-invalid': isValid !== undefined,
        'aria-labelledby': doesNotRenderLabel ? undefined : id,
        'aria-label': doesNotRenderLabel ? renderString(field.label || 'inferno-formlib--InputField', options && options.lang, 'TextArea Field') : undefined,
        'aria-required': field._isRequired ? field._isRequired : undefined
      };
      return inferno.normalizeProps(inferno.createComponentVNode(2, infernoBootstrap.Input, _objectSpread({
        "type": "textarea",
        "id": generateId(namespace, '__Field'),
        "name": inputName,
        "placeholder": renderString(field.placeholder, options && options.lang),
        "readOnly": field.readOnly,
        "value": this.state.value,
        "valid": isValid
      }, ariaLabels, {
        "onInput": this.didGetInput,
        "onChange": this.didGetChange
      })));
    }
  }]);

  return TextAreaWidget;
}(inferno.Component);
new componentRegistry.Adapter({
  implements: IInputFieldWidget,
  adapts: isomorphicSchema.interfaces.ITextAreaField,
  Component: TextAreaWidget
});

var DragNDrop =
/*#__PURE__*/
function (_Component) {
  _inherits(DragNDrop, _Component);

  function DragNDrop(props) {
    var _this;

    _classCallCheck(this, DragNDrop);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(DragNDrop).call(this, props));
    _this.state = {
      inDragCount: 0,
      isDragging: false // I need to bind these event handlers to this, and
      // I only want to do it once to make sure they can be
      // removed properly

    };
    _this.setDrag = _this.setDrag.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.onDocumentDrag = _this.onDocumentDrag.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.onDocumentDragEnd = _this.onDocumentDragEnd.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.onDragEnter = _this.onDragEnter.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.onDragLeave = _this.onDragLeave.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.onDrop = _this.onDrop.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.onDragOver = _this.onDragOver.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.componentDidMount = _this.componentDidMount.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.onDragEnd = _this.onDragEnd.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    return _this;
  }

  _createClass(DragNDrop, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      // TODO: Should we only register a single event handler here which
      // triggers dragStart? This is just a bit weird.
      this.dragStart();
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this._unregisterEventHandlers();
    }
  }, {
    key: "setDrag",
    value: function setDrag(change) {
      var inDragCount = this.state.inDragCount + change;

      if (inDragCount >= 0) {
        this.setState({
          inDragCount: inDragCount
        });
      }
    }
  }, {
    key: "element",
    value: function element() {
      return this.dropArea;
    }
  }, {
    key: "onDocumentDrag",
    value: function onDocumentDrag(e) {
      if (!this.state.isDragging) {
        this.setState({
          isDragging: true
        });
      }
    }
  }, {
    key: "onDocumentDragEnd",
    value: function onDocumentDragEnd(e) {
      if (this.state.isDragging && !e.relatedTarget) {
        this.setState({
          isDragging: false
        });
      }
    }
  }, {
    key: "onDragEnter",
    value: function onDragEnter(e) {
      e.preventDefault();
      this.setDrag(1);
    }
  }, {
    key: "onDragLeave",
    value: function onDragLeave(e) {
      this.setDrag(-1);
    }
  }, {
    key: "onDragEnd",
    value: function onDragEnd(e) {
      this._unregisterEventHandlers();
    }
  }, {
    key: "onDrop",
    value: function onDrop(e) {
      e.preventDefault();
      this.setState({
        inDragCount: 0,
        isDragging: false
      });

      if (this.props.multiFile) {
        var files = e.dataTransfer.files;
        this.props.onDrop(files);
      } else {
        var file = e.dataTransfer.files[0]; // console.log("[DragNDrop] file:", file.name);

        this.props.onDrop(file, file.name);
      } // TODO: Why did we do this? It disables drag'n'drop
      // this._unregisterEventHandlers()

    }
  }, {
    key: "onDragOver",
    value: function onDragOver(e) {
      e.preventDefault();
    }
  }, {
    key: "dragStart",
    value: function dragStart() {
      this._registerEventHandlers();
    }
  }, {
    key: "_registerEventHandlers",
    value: function _registerEventHandlers() {
      var el = this.element();

      if (el) {
        el.addEventListener('dragenter', this.onDragEnter);
        el.addEventListener('dragleave', this.onDragLeave);
        el.addEventListener('dragover', this.onDragOver);
        el.addEventListener('drop', this.onDrop);
        el.addEventListener('dragend', this.onDragEnd);
      }

      document.addEventListener("dragenter", this.onDocumentDrag);
      document.addEventListener("dragleave", this.onDocumentDragEnd);
    }
  }, {
    key: "_unregisterEventHandlers",
    value: function _unregisterEventHandlers() {
      var el = this.element();

      if (el) {
        el.removeEventListener('dragenter', this.onDragEnter);
        el.removeEventListener('dragleave', this.onDragLeave);
        el.removeEventListener('dragover', this.onDragOver);
        el.removeEventListener('drop', this.onDrop);
        el.removeEventListener('dragend', this.onDragEnd);
      }

      document.removeEventListener("dragenter", this.ondDrag);
      document.removeEventListener("dragleave", this.onDocumentDragEnd);
    }
  }, {
    key: "render",
    value: function render() {
      var _this2 = this;

      var cls = classnames('DragNDroppable', {
        'DragNDroppable--drag-hover': this.state.inDragCount > 0,
        'DragNDroppable--drag': this.state.isDragging
      });
      return inferno.createVNode(1, "div", cls, this.props.children, 0, null, null, function (el) {
        _this2.dropArea = el;
      });
    }
  }]);

  return DragNDrop;
}(inferno.Component);

var FileUploadWidget =
/*#__PURE__*/
function (_Component) {
  _inherits(FileUploadWidget, _Component);

  function FileUploadWidget(props) {
    var _this;

    _classCallCheck(this, FileUploadWidget);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(FileUploadWidget).call(this, props));
    _this.state = {
      progress: undefined,
      errMsg: undefined
    };
    _this.didClick = _this.didClick.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.doUpload = _this.doUpload.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.didGetProgress = _this.didGetProgress.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.doClearError = _this.doClearError.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    return _this;
  }

  _createClass(FileUploadWidget, [{
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      if (nextProps.value !== undefined && this.state.progress !== undefined) {
        this.setState({
          progress: undefined,
          errMsg: undefined
        });
      }
    }
  }, {
    key: "didClick",
    value: function didClick(e) {
      var _this2 = this;

      e && e.preventDefault();

      if (this.props.onClick) {
        this.props.onClick(e);
      } else {
        var input = document.createElement('input');
        input.type = 'file';

        input.onchange = function (e) {
          _this2.doUpload(e.target.files[0]);
        };

        input.click();
      }
    }
  }, {
    key: "doClearError",
    value: function doClearError(e) {
      e.preventDefault();
      this.setState({
        progress: undefined,
        errMsg: undefined
      });
    }
  }, {
    key: "didGetProgress",
    value: function didGetProgress(perc) {
      this.setState({
        progress: perc,
        errMsg: undefined
      });
    }
  }, {
    key: "doUpload",
    value: function doUpload(uploadObj) {
      var _this3 = this;

      var file;

      if (uploadObj instanceof FileList) {
        // Upload first file in list
        // TODO: Should we support multiple file uploads?
        file = uploadObj[0];
      } else if (uploadObj instanceof File) {
        file = uploadObj;
      } // Fire change event on currently active element to trigger update of value


      this._inputEl.focus();

      this.setState({
        progress: 0
      });
      var fileUploadUtil = registry.getUtility(IFileUploadUtil, this.props.uploadUtilName || 'Image');
      fileUploadUtil.upload(file, this.didGetProgress).then(function (data) {
        _this3.setState({
          progress: 100
        });

        _this3.props.onChange(_this3.props.name, data);
      }).catch(function (e) {
        _this3.setState({
          errMsg: e.message
        });
      }); // TODO: Handle error
    }
  }, {
    key: "renderProgress",
    value: function renderProgress() {
      return inferno.createVNode(1, "div", "InfernoFormlib-FileUploadWidget", inferno.createVNode(1, "div", "Uploading", inferno.createVNode(1, "div", "ProgressContainer", inferno.createVNode(1, "div", "ProgressBar", null, 1, {
        "style": {
          width: this.state.progress + '%'
        }
      }), 2), 2), 2);
    }
  }, {
    key: "renderError",
    value: function renderError() {
      return inferno.createVNode(1, "div", "InfernoFormlib-FileUploadWidget", inferno.createVNode(1, "div", "Error", inferno.createVNode(1, "div", "ErrorContainer", [inferno.createVNode(1, "p", null, this.state.errMsg, 0), inferno.createComponentVNode(2, infernoBootstrap.Button, {
        "onClick": this.doClearError,
        "size": "sm",
        children: "Try again!"
      })], 4), 2), 2);
    }
  }, {
    key: "renderEmpty",
    value: function renderEmpty() {
      var _this4 = this;

      return inferno.createVNode(1, "div", "InfernoFormlib-FileUploadWidget", [!this.props.value && inferno.createComponentVNode(2, DragNDrop, {
        "onDrop": this.doUpload,
        children: [!this.props.hide && inferno.createVNode(1, "span", "placeholder", this.props.placeholder, 0), !this.props.hide && inferno.createVNode(64, "input", "form-control-file", null, 1, {
          "id": this.props.id,
          "name": this.props.name,
          "type": "file",
          "aria-label": this.props.name || 'file',
          "onClick": this.didClick,
          "onChange": function onChange(e) {
            e.preventDefault();

            _this4.doUpload(e.target.files[0]);
          }
        }, null, function (el) {
          return _this4._inputEl = el;
        })]
      }), this.props.value && this.props.children], 0);
    }
  }, {
    key: "render",
    value: function render() {
      if (this.props.value) {
        return this.props.children;
      } else if (this.state.errMsg) {
        return this.renderError();
      } else if (this.state.progress !== undefined) {
        return this.renderProgress();
      } else {
        return this.renderEmpty();
      }
    }
  }]);

  return FileUploadWidget;
}(inferno.Component);

function mapToCssModules(className, cssModule) {
  if (!cssModule) return className;
  return className.split(' ').map(function (c) {
    return cssModule[c] || c;
  }).join(' ');
}
/*
const propTypes = {
  children: PropTypes.node,
  bar: PropTypes.bool,
  multi: PropTypes.bool,
  tag: PropTypes.string,
  value: PropTypes.oneOfType([
    PropTypes.string,
    PropTypes.number,
  ]),
  max: PropTypes.oneOfType([
    PropTypes.string,
    PropTypes.number,
  ]),
  animated: PropTypes.bool,
  striped: PropTypes.bool,
  color: PropTypes.string,
  className: PropTypes.string,
  barClassName: PropTypes.string,
  cssModule: PropTypes.object,
};
*/

var defaultProps = {
  tag: 'div',
  value: 0,
  max: 100
};

var Progress = function Progress(props) {
  var children = props.children,
      className = props.className,
      barClassName = props.barClassName,
      cssModule = props.cssModule,
      value = props.value,
      max = props.max,
      animated = props.animated,
      striped = props.striped,
      color = props.color,
      bar = props.bar,
      multi = props.multi,
      Tag = props.tag,
      attributes = _objectWithoutProperties(props, ["children", "className", "barClassName", "cssModule", "value", "max", "animated", "striped", "color", "bar", "multi", "tag"]);

  var percent = toNumber(value) / toNumber(max) * 100;
  var progressClasses = mapToCssModules(classnames(className, 'progress'), cssModule);
  var progressBarClasses = mapToCssModules(classnames('progress-bar', bar ? className || barClassName : barClassName, animated ? 'progress-bar-animated' : null, color ? "bg-".concat(color) : null, striped || animated ? 'progress-bar-striped' : null), cssModule);
  var ProgressBar = multi ? children : inferno.createVNode(1, "div", progressBarClasses, children, 2, {
    "style": {
      width: "".concat(percent, "%")
    },
    "role": "progressbar",
    "aria-valuenow": value,
    "aria-valuemin": "0",
    "aria-valuemax": max
  });

  if (bar) {
    return ProgressBar;
  }

  return inferno.normalizeProps(inferno.createComponentVNode(2, Tag, _objectSpread({}, attributes, {
    "className": progressClasses,
    "children": ProgressBar
  })));
};

Progress.defaultProps = defaultProps;

function ProgressOverlay(props) {
  return inferno.createVNode(1, "div", "InfernoFormlib-ProgressOverlay", inferno.createVNode(1, "div", "InfernoFormlib-ProgressBarOut", inferno.createComponentVNode(2, Progress, {
    "striped": true,
    "value": props.progress,
    "max": 100
  }), 2), 2);
}

function CustomWidget(props) {
  // Abstract component used to provide custom widgets
  return null;
}

/*

    Components and helper functions for sending feedback to user about validation errors.

*/

function Label(props) {
  return inferno.createComponentVNode(2, _bs_Label, {
    "id": props.id,
    children: renderString(props.children, props.options && props.options.lang)
  });
}

function HelpMsg(_ref, context) {
  var field = _ref.field,
      text = _ref.text,
      id = _ref.id,
      required = _ref.required,
      _ref$options = _ref.options,
      options = _ref$options === void 0 ? {} : _ref$options;
  // don't render if nothing to show
  if (!text && !required) return null;
  var outp = [];
  if (text) outp.push(renderVariables(field, renderString(text, options.lang)));
  if (required) outp.push(renderVariables(field, renderString(isomorphicSchema.i18n('isomorphic-schema--field_required', '(required)'), options.lang, '(required)')));

  if (context && context.renderHelpAsHtml) {
    return inferno.createComponentVNode(2, FormText, {
      "className": "text-muted",
      "for": id,
      "dangerouslySetInnerHTML": {
        __html: outp.join(' ')
      }
    });
  } else {
    return inferno.createComponentVNode(2, FormText, {
      "className": "text-muted",
      "for": id,
      children: outp.join(' ')
    });
  }
}

function unpackInvariantErrors(validationError, namespace) {
  if (validationError === undefined) return;
  var invariantError;

  if (Array.isArray(validationError.invariantErrors)) {
    var dotName = namespace.join('.');
    var tmpInvErrs = validationError.invariantErrors.filter(function (invErr) {
      // Pattern match field name of error with current namespace to see if it is a match
      return invErr.fields.reduce(function (prev, curr) {
        return prev || curr === dotName;
      }, false);
    });

    if (tmpInvErrs.length > 1) {
      invariantError = {
        message: 'There are several form value errors here',
        i18nLabel: 'inferno-formlib--multiple_invariant_errors'
      };
    } else {
      invariantError = tmpInvErrs[0];
    }
  }

  return invariantError;
}

var ErrorMsg =
/*#__PURE__*/
function (_Component) {
  _inherits(ErrorMsg, _Component);

  function ErrorMsg() {
    _classCallCheck(this, ErrorMsg);

    return _possibleConstructorReturn(this, _getPrototypeOf(ErrorMsg).apply(this, arguments));
  }

  _createClass(ErrorMsg, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      infernoAnimation.animateOnAdd(infernoExtras.findDOMNode(this), 'InfernoFormlib-ErrorMsg--Animation');
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      infernoAnimation.animateOnRemove(infernoExtras.findDOMNode(this), 'InfernoFormlib-ErrorMsg--Animation');
    }
  }, {
    key: "render",
    value: function render(_ref2) {
      var field = _ref2.field,
          validationError = _ref2.validationError,
          _ref2$options = _ref2.options,
          options = _ref2$options === void 0 ? {} : _ref2$options;
      return inferno.createComponentVNode(2, FormFeedback, {
        children: renderVariables(field, renderString(validationError.i18nLabel, options.lang, validationError.message))
      });
    }
  }]);

  return ErrorMsg;
}(inferno.Component);

/*
    PROPS:
    - animation: animation css class prefix
    - submitted: bool, has been submitted
    - field:  isomorphic-schema field validator object
    - errors: isomorphic-schema field and server error object { fieldErrors, serverErrors } or undefined if no errors
    - id:     unique id of field
*/

var Row =
/*#__PURE__*/
function (_Component) {
  _inherits(Row, _Component);

  function Row() {
    _classCallCheck(this, Row);

    return _possibleConstructorReturn(this, _getPrototypeOf(Row).apply(this, arguments));
  }

  _createClass(Row, [{
    key: "componentDidMount",
    // TODO: Add animation support
    // support required
    value: function componentDidMount() {
      if (this.props.formIsMounted) {
        infernoAnimation.animateOnAdd(infernoExtras.findDOMNode(this), 'InfernoFormlib-Row--Animation');
      }
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      infernoAnimation.animateOnRemove(infernoExtras.findDOMNode(this), 'InfernoFormlib-Row--Animation');
    }
  }, {
    key: "render",
    value: function render(_ref) {
      var validationError = _ref.validationError,
          adapter = _ref.adapter,
          submitted = _ref.submitted,
          options = _ref.options,
          children = _ref.children,
          id = _ref.id;
      var field = adapter.context;
      var invariantError = unpackInvariantErrors(this.props.validationError, this.props.namespace);
      return inferno.createComponentVNode(2, infernoBootstrap.FormGroup, {
        "id": this.props.namespace.join('.') + '__Row',
        children: [field.label && inferno.createComponentVNode(2, Label, {
          "id": id,
          "options": options,
          children: field.label
        }), inferno.createVNode(1, "div", "InfernoFormlib-RowFieldContainer", children, 0), validationError ? inferno.createComponentVNode(2, ErrorMsg, {
          "field": field,
          "validationError": validationError,
          "submitted": submitted,
          "options": options
        }) : null, invariantError ? inferno.createComponentVNode(2, ErrorMsg, {
          "field": field,
          "validationError": invariantError,
          "submitted": submitted,
          "options": options
        }) : null, inferno.createComponentVNode(2, HelpMsg, {
          "field": field,
          "text": field.help,
          "required": field._isRequired && !validationError,
          "options": options
        })]
      });
    }
  }]);

  return Row;
}(inferno.Component);

new componentRegistry.Adapter({
  implements: IFormRowWidget,
  adapts: isomorphicSchema.interfaces.IBaseField,
  Component: Row,
  doesNotRenderLabel: function doesNotRenderLabel() {
    return this.context.label ? false : true;
  }
});

var ObjectRow =
/*#__PURE__*/
function (_Component2) {
  _inherits(ObjectRow, _Component2);

  function ObjectRow() {
    _classCallCheck(this, ObjectRow);

    return _possibleConstructorReturn(this, _getPrototypeOf(ObjectRow).apply(this, arguments));
  }

  _createClass(ObjectRow, [{
    key: "componentDidMount",
    // TODO: Add animation support
    // support required
    value: function componentDidMount() {
      if (this.props.formIsMounted) {
        infernoAnimation.animateOnAdd(infernoExtras.findDOMNode(this), 'InfernoFormlib-Row--Animation');
      }
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      infernoAnimation.animateOnRemove(infernoExtras.findDOMNode(this), 'InfernoFormlib-Row--Animation');
    }
  }, {
    key: "render",
    value: function render(_ref2) {
      var validationError = _ref2.validationError,
          adapter = _ref2.adapter,
          submitted = _ref2.submitted,
          options = _ref2.options,
          children = _ref2.children,
          id = _ref2.id;
      var field = adapter.context;
      var invariantError = unpackInvariantErrors(this.props.validationError, this.props.namespace);
      return inferno.createComponentVNode(2, infernoBootstrap.FormGroup, {
        "className": "InfernoFormlib-ObjectRow",
        children: [field.label && inferno.createComponentVNode(2, Label, {
          "id": id,
          "options": options,
          children: field.label
        }), validationError ? inferno.createComponentVNode(2, ErrorMsg, {
          "field": field,
          "validationError": validationError,
          "submitted": submitted,
          "options": options
        }) : null, invariantError ? inferno.createComponentVNode(2, ErrorMsg, {
          "field": field,
          "validationError": invariantError,
          "submitted": submitted,
          "options": options
        }) : null, field.help ? inferno.createComponentVNode(2, HelpMsg, {
          "field": field,
          "text": field.help,
          "required": field._isRequired,
          "options": options
        }) : null, inferno.createVNode(1, "div", "InfernoFormlib-RowFieldContainer", children, 0)]
      });
    }
  }]);

  return ObjectRow;
}(inferno.Component);

new componentRegistry.Adapter({
  implements: IFormRowWidget,
  adapts: isomorphicSchema.interfaces.IObjectField,
  Component: ObjectRow,
  doesNotRenderLabel: function doesNotRenderLabel() {
    return this.context.label ? false : true;
  }
});

var ListRow =
/*#__PURE__*/
function (_Component3) {
  _inherits(ListRow, _Component3);

  function ListRow() {
    _classCallCheck(this, ListRow);

    return _possibleConstructorReturn(this, _getPrototypeOf(ListRow).apply(this, arguments));
  }

  _createClass(ListRow, [{
    key: "componentDidMount",
    // TODO: Add animation support
    // support required
    value: function componentDidMount() {
      if (this.props.formIsMounted) {
        infernoAnimation.animateOnAdd(infernoExtras.findDOMNode(this), 'InfernoFormlib-Row--Animation');
      }
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      infernoAnimation.animateOnRemove(infernoExtras.findDOMNode(this), 'InfernoFormlib-Row--Animation');
    }
  }, {
    key: "render",
    value: function render(_ref3) {
      var validationError = _ref3.validationError,
          adapter = _ref3.adapter,
          submitted = _ref3.submitted,
          options = _ref3.options,
          children = _ref3.children,
          id = _ref3.id;
      var field = adapter.context;
      var invariantError = unpackInvariantErrors(this.props.validationError, this.props.namespace);
      return inferno.createComponentVNode(2, infernoBootstrap.FormGroup, {
        "className": "InfernoFormlib-ListRow",
        children: [field.label && inferno.createComponentVNode(2, Label, {
          "id": id,
          "options": options,
          children: field.label
        }), validationError ? inferno.createComponentVNode(2, ErrorMsg, {
          "field": field,
          "validationError": validationError,
          "submitted": submitted,
          "options": options
        }) : null, invariantError ? inferno.createComponentVNode(2, ErrorMsg, {
          "field": field,
          "validationError": invariantError,
          "submitted": submitted,
          "options": options
        }) : null, field.help ? inferno.createComponentVNode(2, HelpMsg, {
          "field": field,
          "text": field.help,
          "required": field._isRequired,
          "options": options
        }) : null, inferno.createVNode(1, "div", "InfernoFormlib-RowFieldContainer", children, 0)]
      });
    }
  }]);

  return ListRow;
}(inferno.Component);

new componentRegistry.Adapter({
  implements: IFormRowWidget,
  adapts: isomorphicSchema.interfaces.IListField,
  Component: ListRow,
  doesNotRenderLabel: function doesNotRenderLabel() {
    return this.context.label ? false : true;
  }
});

var CheckboxRow =
/*#__PURE__*/
function (_Component4) {
  _inherits(CheckboxRow, _Component4);

  function CheckboxRow() {
    _classCallCheck(this, CheckboxRow);

    return _possibleConstructorReturn(this, _getPrototypeOf(CheckboxRow).apply(this, arguments));
  }

  _createClass(CheckboxRow, [{
    key: "componentDidMount",
    // TODO: Add animation support
    // support required
    value: function componentDidMount() {
      if (this.props.formIsMounted) {
        infernoAnimation.animateOnAdd(infernoExtras.findDOMNode(this), 'InfernoFormlib-Row--Animation');
      }
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      infernoAnimation.animateOnRemove(infernoExtras.findDOMNode(this), 'InfernoFormlib-Row--Animation');
    }
  }, {
    key: "render",
    value: function render(_ref4) {
      var validationError = _ref4.validationError,
          adapter = _ref4.adapter,
          submitted = _ref4.submitted,
          options = _ref4.options,
          children = _ref4.children,
          id = _ref4.id;
      var field = adapter.context;
      var invariantError = unpackInvariantErrors(this.props.validationError, this.props.namespace);
      return inferno.createComponentVNode(2, infernoBootstrap.FormGroup, {
        "id": this.props.namespace.join('.') + '__Row',
        "check": true,
        children: [inferno.createVNode(1, "div", "InfernoFormlib-RowFieldContainer", inferno.createComponentVNode(2, infernoBootstrap.Label, {
          "id": id,
          "check": true,
          children: [children, renderString(field.label, options && options.lang)]
        }), 2), validationError ? inferno.createComponentVNode(2, ErrorMsg, {
          "field": field,
          "validationError": validationError,
          "submitted": submitted,
          "options": options
        }) : null, invariantError ? inferno.createComponentVNode(2, ErrorMsg, {
          "field": field,
          "validationError": invariantError,
          "submitted": submitted,
          "options": options
        }) : null, inferno.createComponentVNode(2, HelpMsg, {
          "field": field,
          "text": field.help,
          "required": field._isRequired,
          "options": options
        })]
      });
    }
  }]);

  return CheckboxRow;
}(inferno.Component);

new componentRegistry.Adapter({
  implements: IFormRowWidget,
  adapts: isomorphicSchema.interfaces.IBoolField,
  Component: CheckboxRow,
  doesNotRenderLabel: function doesNotRenderLabel() {
    return this.context.label ? false : true;
  }
});

var widgets = {
  ActionBar: ActionBar,
  AutoCompleteWidget: AutoCompleteBaseWidget,
  AnyOfWidget: AnyOf,
  CheckboxWidget: CheckboxWidget,
  DynamicSelectWidget: DynamicSelectFieldWidget,
  FileUploadWidget: FileUploadWidget,
  InputWidget: InputWidget$2,
  ListWidget: ListFieldWidget,
  MultiSelectWidget: MultiSelectFieldWidget,
  ObjectWidget: ObjectFieldWidget,
  PasswordWidget: PasswordWidget,
  SelectWidget: SelectFieldWidget,
  TextAreaWidget: TextAreaWidget,
  SelectAsyncBaseWidget: SelectAsyncBaseWidget,
  DragNDrop: DragNDrop,
  ProgressOverlay: ProgressOverlay
};

exports.CustomWidget = CustomWidget;
exports.FormRows = FormRows;
exports.interfaces = interfaces;
exports.widgets = widgets;
exports.CheckboxRow = CheckboxRow;
exports.ObjectRow = ObjectRow;
exports.Row = Row;
exports.ErrorMsg = ErrorMsg;
exports.HelpMsg = HelpMsg;
exports.Label = Label;
exports.unpackInvariantErrors = unpackInvariantErrors;
exports.renderString = renderString;
exports.renderVariables = renderVariables;
exports.getElOffset = getElOffset;
exports.escapeIdSelector = escapeIdSelector;
exports.generateId = generateId;
exports.throttle = throttle;
//# sourceMappingURL=index.cjs.js.map
