'use strict';
/*

    To use this input widget adapter you need to register it with your
    adapter registry or include as a custom field.

        new Adapter({
            implements: IInputFieldWidget,
            adapts: IYourCustomField,
            Component: AutoCompleteBaseWidget,
        })

    or

        <CustomWidget propPath='customPropName' fieldWidget={AutoCompleteWidget} />

*/

import _classCallCheck from "@babel/runtime/helpers/classCallCheck";
import _createClass from "@babel/runtime/helpers/createClass";
import _possibleConstructorReturn from "@babel/runtime/helpers/possibleConstructorReturn";
import _getPrototypeOf from "@babel/runtime/helpers/getPrototypeOf";
import _inherits from "@babel/runtime/helpers/inherits";
import _assertThisInitialized from "@babel/runtime/helpers/assertThisInitialized";
import { createVNode, createComponentVNode } from "inferno";
import { Component } from 'inferno';
import { findDOMNode } from 'inferno-extras';
import { animateOnAdd, animateOnRemove } from 'inferno-animation';
import classnames from 'classnames';
import { Input, Dropdown, DropdownMenu, DropdownItem } from 'inferno-bootstrap';
import { Target } from 'inferno-popper';
import { renderString } from './common';
import { generateId, throttle } from './utils'; // Placeholder

var AutocompleteItem =
/*#__PURE__*/
function (_Component) {
  _inherits(AutocompleteItem, _Component);

  function AutocompleteItem(props) {
    var _this;

    _classCallCheck(this, AutocompleteItem);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(AutocompleteItem).call(this, props));
    _this.didClick = _this.didClick.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    return _this;
  }

  _createClass(AutocompleteItem, [{
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      if (nextProps.isMarked && !this.props.isMarked) {
        // Check if it is visible
        var domNode = findDOMNode(this);
        var currBottom = domNode.offsetTop + domNode.offsetHeight;

        if (currBottom > domNode.parentNode.offsetHeight + domNode.parentNode.scrollTop) {
          domNode.parentNode.scrollTop = currBottom - domNode.parentNode.offsetHeight;
        } else if (domNode.offsetTop < domNode.parentNode.scrollTop) {
          domNode.parentNode.scrollTop = domNode.offsetTop;
        }
      }
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      animateOnAdd(findDOMNode(this), 'InfernoFormlib-AutocompleteItem--Animation');
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      animateOnRemove(findDOMNode(this), 'InfernoFormlib-AutocompleteItem--Animation');
    }
  }, {
    key: "didClick",
    value: function didClick(e) {
      e.preventDefault();
      this.props.onSelect(this.props.value);
    }
  }, {
    key: "render",
    value: function render() {
      var cls = {
        'InfernoFormlib-AutocompleteItem': true,
        'InfernoFormlib-AutocompleteItem--Highlight': this.props.isMarked
      };
      return createComponentVNode(2, DropdownItem, {
        "className": classnames(cls),
        "onClick": this.didClick,
        children: this.props.value.title
      });
    }
  }]);

  return AutocompleteItem;
}(Component);

function _getOptionsAsync(value, options, context) {
  var currentMarked = this.state.options[this.state.markAtIndex];
  return this.props.adapter.context.getOptionsAsync(value, options, context).then(function (results) {
    // Maintain selection when updating list if possible
    var markAtIndex = currentMarked && results.reduce(function (curr, next, index) {
      return curr === undefined && next.name === currentMarked.name ? index : curr;
    }, undefined);
    return {
      options: results,
      isOpen: true,
      markAtIndex: markAtIndex || 0 // Default to select top most option

    };
  }).catch(function () {
    return {
      options: [{
        name: value,
        title: 'Fel när data laddades...'
      }],
      isOpen: true,
      markAtIndex: undefined
    };
  });
}

function TypeAhead(props) {
  if (!props.markedValue) {
    return null;
  } else {
    var text = props.text;
    var suffix = props.markedValue.title.toLowerCase().indexOf(text.toLowerCase()) === 0 ? props.markedValue.title.substr(text.length) : '';
    return createVNode(1, "div", "InfernoFormlib-AutoComplete-TypeAhead", [createVNode(1, "span", "InfernoFormlib-AutoComplete-TypeAheadPrefix", text, 0), createVNode(1, "span", "InfernoFormlib-AutoComplete-TypeAheadSuffix", suffix, 0)], 4);
  }
}

var _actionKeys = {
  ArrowDown: true,
  ArrowUp: true,
  Enter: true
};

var AutoCompleteBaseWidget =
/*#__PURE__*/
function (_Component2) {
  _inherits(AutoCompleteBaseWidget, _Component2);

  function AutoCompleteBaseWidget(props) {
    var _this2;

    _classCallCheck(this, AutoCompleteBaseWidget);

    _this2 = _possibleConstructorReturn(this, _getPrototypeOf(AutoCompleteBaseWidget).call(this, props));
    var field = _this2.props.adapter.context; // Throttle API-calls (ms)

    _this2._throttleTimeout = 300;
    _this2.state = {
      text: props.value && _this2.toText(field.toFormattedString(props.value)),
      isOpen: false,
      options: [],
      markAtIndex: undefined
    };
    _this2.didGetHotKey = _this2.didGetHotKey.bind(_assertThisInitialized(_assertThisInitialized(_this2)));
    _this2.didGetInput = _this2.didGetInput.bind(_assertThisInitialized(_assertThisInitialized(_this2)));
    _this2.didGetChange = _this2.didGetChange.bind(_assertThisInitialized(_assertThisInitialized(_this2)));
    _this2.didSelect = _this2.didSelect.bind(_assertThisInitialized(_assertThisInitialized(_this2)));
    _this2.doToggle = _this2.doToggle.bind(_assertThisInitialized(_assertThisInitialized(_this2)));
    return _this2;
  }

  _createClass(AutoCompleteBaseWidget, [{
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      // TODO: How do we supply these because they could be used to get options
      var options = undefined;
      var context = undefined;
      var field = this.props.adapter.context;
      this.setState({
        text: nextProps.value && this.toText(field.toFormattedString(nextProps.value)),
        value: nextProps.value
      });
    }
  }, {
    key: "toText",
    value: function toText(value) {
      return value && value.title;
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      document.addEventListener('keydown', this.didGetHotKey, false);
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      document.removeEventListener('keydown', this.didGetHotKey);
    }
  }, {
    key: "doToggle",
    value: function doToggle(e) {
      e && e.preventDefault();
      this.setState({
        isOpen: !this.state.isOpen
      });
    }
  }, {
    key: "didGetHotKey",
    value: function didGetHotKey(e) {
      // Use for navigation and enter only!
      console.log('DID PRESS: ' + e);

      if (_actionKeys[e.key] && this.state.markAtIndex !== undefined) {
        // So we got a hotkey
        var domEl = findDOMNode(this._inputNode);

        if (domEl === e.target) {
          // And it was fired in our input box
          if (e.key === 'ArrowUp') {
            this.setState({
              markAtIndex: this.state.markAtIndex <= 0 ? this.state.options.length - 1 : this.state.markAtIndex - 1
            });
            e.preventDefault();
          } else if (e.key === 'ArrowDown') {
            this.setState({
              markAtIndex: this.state.markAtIndex >= this.state.options.length - 1 ? 0 : this.state.markAtIndex + 1
            });
            e.preventDefault();
          } else if (e.key === 'Enter') {
            if (this.state.markAtIndex !== undefined) {
              this.didSelect(this.state.options[this.state.markAtIndex]);
            }

            e.preventDefault();
          }
        }
      }
    }
  }, {
    key: "didGetInput",
    value: function didGetInput(e) {
      var _this3 = this;

      // TODO: How do we supply these because they could be used to get options
      var options = {
        search: e.target.value
      };

      if (e.target.value !== this.state.text && e.target.value.length > 1) {
        var __timerName__ = '_optionsTimer';
        throttle.call(this, this._throttleTimeout, __timerName__).then(function (timer) {
          var context = undefined;

          _getOptionsAsync.call(_this3, undefined, options, context).then(function (res) {
            // Only setState if this is the latest executing timer
            if (_this3[__timerName__] === timer) {
              _this3.setState(res);
            }
          });
        });
      }

      this.setState({
        text: e.target.value,
        markAtIndex: e.target.value ? this.state.markAtIndex : undefined
      });
    }
  }, {
    key: "didGetChange",
    value: function didGetChange(e) {// Not doing anything for now
    }
  }, {
    key: "didSelect",
    value: function didSelect(value) {
      var field = this.props.adapter.context;
      this.setState({
        options: [],
        isOpen: false,
        markAtIndex: undefined
      });
      this.props.onChange(this.props.propName, field.fromString(value)); // Blur focus to emphasize that selection has been done

      findDOMNode(this._inputNode).blur();
    }
  }, {
    key: "renderInput",
    value: function renderInput(_ref) {
      var _this4 = this;

      var inputName = _ref.inputName,
          namespace = _ref.namespace,
          options = _ref.options;
      var field = this.props.adapter.context;
      var cls = {
        "form-control": true,
        "form-control-danger": this.props.validationError
      };
      return createComponentVNode(2, Input, {
        "type": "text",
        "id": generateId(namespace, '__Field'),
        "name": inputName,
        "className": classnames(cls),
        "autocomplete": "off",
        "placeholder": renderString(field.placeholder, options && options.lang),
        "readOnly": field.readOnly,
        "value": this.state.text,
        "onChange": this.didGetChange,
        "onInput": this.didGetInput
      }, null, function (node) {
        return _this4._inputNode = node;
      });
    }
  }, {
    key: "renderTypeAhead",
    value: function renderTypeAhead() {
      if (this.state.text) {
        return createComponentVNode(2, TypeAhead, {
          "text": this.state.text,
          "markedValue": this.state.options[this.state.markAtIndex]
        });
      }
    }
  }, {
    key: "renderDropdownMenu",
    value: function renderDropdownMenu() {
      var _this5 = this;

      // Only show options if we have entered text
      var options = this.state.text ? this.state.options : [];
      return createComponentVNode(2, DropdownMenu, {
        "className": "InfernoFormlib-AutoCompleteItemContainer",
        children: options.map(function (item, index) {
          return createComponentVNode(2, AutocompleteItem, {
            "value": item,
            "isMarked": index === _this5.state.markAtIndex,
            "onSelect": _this5.didSelect
          }, index);
        })
      });
    }
  }, {
    key: "render",
    value: function render(props) {
      return createComponentVNode(2, Dropdown, {
        "className": "InfernoFormlib-AutoCompleteField",
        "isOpen": this.state.isOpen,
        "toggle": this.doToggle,
        children: [createComponentVNode(2, Target, {
          children: [this.renderInput(props), this.renderTypeAhead()]
        }), this.renderDropdownMenu()]
      });
    }
  }]);

  return AutoCompleteBaseWidget;
}(Component);

export default AutoCompleteBaseWidget;
//# sourceMappingURL=AutoComplete.js.map