'use strict';
/*

    To use this input widget adapter you need to register it with your
    adapter registry.

*/

import _objectSpread from "@babel/runtime/helpers/objectSpread";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _classCallCheck from "@babel/runtime/helpers/classCallCheck";
import _createClass from "@babel/runtime/helpers/createClass";
import _possibleConstructorReturn from "@babel/runtime/helpers/possibleConstructorReturn";
import _getPrototypeOf from "@babel/runtime/helpers/getPrototypeOf";
import _inherits from "@babel/runtime/helpers/inherits";
import _assertThisInitialized from "@babel/runtime/helpers/assertThisInitialized";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
import { createVNode, createComponentVNode, normalizeProps } from "inferno";
import { Component } from 'inferno';
import { findDOMNode } from 'inferno-extras';
import { Adapter } from 'component-registry';
import { interfaces } from 'isomorphic-schema';
import { IInputFieldWidget } from '../interfaces';
import classNames from 'classnames';
import { renderString } from './common';
import { generateId, escapeIdSelector } from './utils';
import classnames from 'classnames';
import { DropdownToggle, DropdownMenu, Input, InputGroup, Popover, PopoverBody, PopoverHeader, Nav, NavItem, NavLink, NavDropdown } from 'inferno-bootstrap';
import { Manager, Target } from 'inferno-popper';
var weekStartsOn = 1;

function isProperDate(inp) {
  return typeof inp === 'string' && inp.length === 10 && !isNaN(new Date(inp).getTime());
}

function toMs(days) {
  return days * 24 * 3600000;
}

function CalendarDay(props) {
  return createVNode(1, "td", classNames("DateFieldItem", {
    "text-muted": !props.isCurrentMonth,
    "DateFieldItem--selected": props.isSelected,
    "DateFieldItem--today": props.isToday
  }), createVNode(1, "a", null, props.dayNr, 0, {
    "href": '#'
  }), 2, {
    "onClick": function onClick(e) {
      e.preventDefault();
      props.onClick(props.value);
    }
  });
}

function getCalendar(date, locale) {
  var today = new Date();
  var firstOfMonth = new Date(date.getUTCFullYear(), date.getUTCMonth(), 1);
  var showDays = [];
  var rowNr = 0; // Get first day to show

  var tmpDay = new Date(firstOfMonth.valueOf() - toMs(firstOfMonth.getUTCDay() ? firstOfMonth.getUTCDay() - weekStartsOn : 7 - weekStartsOn));

  while (rowNr === 0 || tmpDay.getUTCMonth() === date.getUTCMonth() || tmpDay.getUTCDay() - weekStartsOn !== 0) {
    showDays.push({
      dayNr: tmpDay.getUTCDate(),
      isCurrentMonth: tmpDay.getUTCMonth() === date.getUTCMonth(),
      isToday: tmpDay.getUTCFullYear() === today.getFullYear() && tmpDay.getUTCMonth() === today.getMonth() && tmpDay.getUTCDate() === today.getDate(),
      value: tmpDay.toISOString().slice(0, 10)
    });
    tmpDay = new Date(tmpDay.valueOf() + toMs(1));

    if ((tmpDay.getUTCDate() - weekStartsOn) % 7 === 0) {
      rowNr++;
    }
  }

  var weekdaysMin = renderString('inferno-formlib--DateField-weekdays', undefined, 'sö_må_ti_on_to_fr_lö').split('_');
  var months = renderString('inferno-formlib--DateField-months', undefined, 'januari_februari_mars_april_maj_juni_juli_augusti_september_oktober_november_december').split('_');
  var cal = {
    monthName: months[date.getUTCMonth()],
    year: date.getUTCFullYear(),
    dayHeaders: weekdaysMin.map(function (item, index) {
      return weekdaysMin[(index + weekStartsOn) % 7]; // rotate weekdays by weekStartsOn
    }),
    days: showDays
  };
  return cal;
}

var Calendar =
/*#__PURE__*/
function (_Component) {
  _inherits(Calendar, _Component);

  function Calendar() {
    var _getPrototypeOf2;

    var _this;

    _classCallCheck(this, Calendar);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _possibleConstructorReturn(this, (_getPrototypeOf2 = _getPrototypeOf(Calendar)).call.apply(_getPrototypeOf2, [this].concat(args)));

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "state", {
      yearSelectOpen: false,
      showSelectYear: undefined
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "doStepYear", function (e, year) {
      e && e.preventDefault();

      _this.setState({
        showSelectYear: year
      });
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "doSelectYear", function (e, year) {
      e && e.preventDefault();
      var showMonth = _this.props.showMonth;

      _this.props.onNavigate(new Date(year, showMonth.month, 2));

      _this.doToggleYearSelect();
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "doToggleYearSelect", function (e) {
      e && e.preventDefault();

      _this.setState({
        yearSelectOpen: !_this.state.yearSelectOpen,
        showSelectYear: _this.props.showMonth.year
      });
    });

    return _this;
  }

  _createClass(Calendar, [{
    key: "renderYearSelect",
    value: function renderYearSelect() {
      var _this2 = this;

      var startYear = this.state.showSelectYear - 4;
      return createVNode(1, "ul", "DateField-YearSelect", [createVNode(1, "a", "DateField-YearSelect-minus", createVNode(1, "i", null, "-", 16), 2, {
        "onClick": function onClick(e) {
          return _this2.doStepYear(e, startYear - 5);
        }
      }), [0, 1, 2, 3, 4, 5, 6, 7, 8].map(function (i) {
        return createVNode(1, "li", "DateField-YearSelect-day", createVNode(1, "a", null, startYear + i, 0, {
          "onClick": function onClick(e) {
            return _this2.doSelectYear(e, startYear + i);
          }
        }), 2);
      }), createVNode(1, "a", "DateField-YearSelect-plus", createVNode(1, "i", null, "+", 16), 2, {
        "onClick": function onClick(e) {
          return _this2.doStepYear(e, startYear + 13);
        }
      })], 0);
    }
  }, {
    key: "render",
    value: function render(props) {
      var showMonth = props.showMonth,
          popoverProps = _objectWithoutProperties(props, ["showMonth"]);

      var tmpDate = new Date(showMonth.year, showMonth.month, 2);
      var currentMonth = new Date(tmpDate);
      var calendar = getCalendar(tmpDate);
      var rows = [];
      var days = [];
      var i = 0;
      calendar.days.forEach(function (day, i) {
        days.push(createComponentVNode(2, CalendarDay, {
          "value": day.value,
          "isSelected": day.value === props.value,
          "isCurrentMonth": day.isCurrentMonth,
          "isToday": day.isToday,
          "dayNr": day.dayNr,
          "onClick": props.onSelect
        }));
        i++;

        if (i % 7 === 0) {
          rows.push(createVNode(1, "tr", null, days, 0));
          days = [];
        } else if (i === calendar.days.length) {
          rows.push(createVNode(1, "tr", null, days, 0));
          days = [];
        }
      });
      return normalizeProps(createComponentVNode(2, Popover, _objectSpread({}, popoverProps, {
        children: [createComponentVNode(2, PopoverHeader, {
          children: createComponentVNode(2, Nav, {
            "style": {
              width: "100%"
            },
            children: [createComponentVNode(2, NavItem, {
              children: createComponentVNode(2, NavLink, {
                "href": "<",
                "onClick": function onClick(e) {
                  e.preventDefault();
                  props.onNavigate(new Date(currentMonth.setMonth(currentMonth.getMonth() - 1)));
                },
                children: '<'
              })
            }), createComponentVNode(2, NavItem, {
              "className": "DateFieldMonth",
              children: createComponentVNode(2, NavDropdown, {
                "isOpen": this.state.yearSelectOpen,
                "toggle": this.doToggleYearSelect,
                children: [createComponentVNode(2, DropdownToggle, {
                  "nav": true,
                  "caret": true,
                  children: [createVNode(1, "small", null, calendar.year, 0), ' ' + calendar.monthName]
                }), createComponentVNode(2, DropdownMenu, {
                  children: createVNode(1, "div", null, this.renderYearSelect(), 0)
                })]
              })
            }), createComponentVNode(2, NavItem, {
              children: createComponentVNode(2, NavLink, {
                "href": ">",
                "onClick": function onClick(e) {
                  e.preventDefault();
                  props.onNavigate(new Date(currentMonth.setMonth(currentMonth.getMonth() + 1)));
                },
                children: '>'
              })
            })]
          })
        }), createComponentVNode(2, PopoverBody, {
          "className": "DateFieldBody",
          children: createVNode(1, "table", null, createVNode(1, "tbody", null, [createVNode(1, "tr", "DateFieldHeaderRow", calendar.dayHeaders.map(function (day) {
            return createVNode(1, "th", "DateFieldHeaderItem", day, 0);
          }), 0), rows], 0), 2, {
            "style": {
              width: "100%"
            }
          })
        }), createVNode(1, "div", "DateFieldFooter", createVNode(1, "a", null, props.value || renderString('inferno-formlib--DateField-select_date', undefined, 'select a date'), 0, {
          "href": "#showSelected",
          "onClick": props.onShowSelected
        }), 2)]
      })));
    }
  }]);

  return Calendar;
}(Component);

var InputWidget =
/*#__PURE__*/
function (_Component2) {
  _inherits(InputWidget, _Component2);

  function InputWidget(props) {
    var _this3;

    _classCallCheck(this, InputWidget);

    _this3 = _possibleConstructorReturn(this, _getPrototypeOf(InputWidget).call(this, props)); // Show the month of the current date value

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this3)), "didClickBody", function (e) {
      if (_this3.state.popoverOpen) {
        var tmpNode = e.target;
        var doesContainTarget = false;

        while (tmpNode) {
          // NOTE: Using className to check if we clicked in the calendar. Should use better test.
          if (findDOMNode(_this3._elInput) === tmpNode || tmpNode.className && tmpNode.className.indexOf('InfernoFormlib-DateFieldCalendar') >= 0) {
            doesContainTarget = true;
            break;
          }

          tmpNode = tmpNode.parentNode;
        }

        if (!doesContainTarget) {
          _this3.doHidePopover();
        }
      }
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this3)), "doUpdateValue", function (newDate) {
      _this3.setState({
        value: newDate
      });

      _this3.props.onChange(_this3.props.propName, newDate);

      _this3.doHidePopover();
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this3)), "doUpdateShowMonth", function (newDate) {
      _this3.setState({
        showMonth: {
          year: newDate.getFullYear(),
          month: newDate.getMonth()
        }
      });
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this3)), "doShowSelected", function (e) {
      e.preventDefault();

      _this3.setState({
        showMonth: {
          year: parseInt(_this3.state.value.slice(0, 4)),
          month: parseInt(_this3.state.value.slice(5, 7)) - 1
        }
      });
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this3)), "didGetInput", function (e) {
      var field = _this3.props.adapter.context;
      var value = field.fromString(e.target.value);

      _this3.setState({
        value: value // popoverOpen: this.state.popoverOpen && isProperDate(value)

      });

      if (value === undefined || isProperDate(value)) {
        _this3.props.onChange(_this3.props.propName, value);
      }
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this3)), "togglePopover", function () {
      if (_this3.state.popoverOpen) {
        _this3.doHidePopover();
      } else {
        _this3.doShowPopover;
      }
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this3)), "doShowPopover", function () {
      if (!_this3.state.popoverOpen) {
        _this3.setState({
          popoverOpen: true
        });

        document.addEventListener('click', _this3.didClickBody);
      }
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this3)), "doHidePopover", function (e) {
      if (_this3.state.popoverOpen) {
        _this3.setState({
          popoverOpen: false
        });

        document.removeEventListener('click', _this3.didClickBody);
      }
    });

    _defineProperty(_assertThisInitialized(_assertThisInitialized(_this3)), "doUnset", function (e) {
      e.preventDefault();

      _this3.props.onChange(_this3.props.propName, undefined);
    });

    var date;

    if (isProperDate(props.value)) {
      date = new Date(props.value);
    } else {
      date = new Date();
    } // Adjust internal date so UTC equals current timezone


    date = new Date(date.valueOf() - date.getTimezoneOffset() * 60000 + 12 * 3600);
    _this3.state = {
      value: props.value,
      popoverOpen: false,
      showMonth: {
        year: date.getUTCFullYear(),
        month: date.getUTCMonth()
      }
    };
    return _this3;
  }

  _createClass(InputWidget, [{
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.doHidePopover();
    }
  }, {
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      if (nextProps.value !== this.props.value) {
        this.setState({
          value: nextProps.value // popoverOpen: this.state.popoverOpen && isProperDate(nextProps.value)

        }); // Update calendar

        var date = isProperDate(nextProps.value) ? new Date(nextProps.value) : new Date(); // Adjust internal date so UTC equals current timezone

        date = new Date(date.valueOf() - date.getTimezoneOffset() * 60000 + 12 * 3600);
        this.doUpdateShowMonth(date);
      }
    }
  }, {
    key: "render",
    value: function render(_ref) {
      var _this4 = this;

      var inputName = _ref.inputName,
          namespace = _ref.namespace,
          options = _ref.options,
          doesNotRenderLabel = _ref.doesNotRenderLabel,
          id = _ref.id;
      var field = this.props.adapter.context;
      var isValid = this.props.validationError || this.props.invariantError ? false : undefined;
      var ariaLabels = {
        'aria-invalid': isValid !== undefined,
        'aria-labelledby': doesNotRenderLabel ? undefined : id,
        'aria-label': doesNotRenderLabel ? renderString(field.label || 'inferno-formlib--DateField', options && options.lang, 'Date Field') : undefined,
        'aria-required': field._isRequired ? field._isRequired : undefined
      };
      var inputId = generateId(namespace, '__Field'); // TODO: onBlur={this.doHidePopover}

      return createComponentVNode(2, Manager, {
        children: [createComponentVNode(2, Target, {
          children: createComponentVNode(2, InputGroup, {
            "className": this.props.className,
            children: [normalizeProps(createComponentVNode(2, Input, _objectSpread({
              "className": this.props.innerClassName,
              "id": inputId,
              "name": inputName,
              "valid": isValid,
              "placeholder": renderString(field.placeholder, options && options.lang),
              "readOnly": field.readOnly,
              "value": field.toFormattedString(this.state.value)
            }, ariaLabels, {
              "onFocus": this.doShowPopover,
              "onBlur": this.didClickBody,
              "onInput": this.didGetInput
            }), null, function (el) {
              return _this4._elInput = el;
            })), this.props.children, createVNode(1, "a", "InfernoFormlib-DateField-unset", renderString('inferno-formlib--DateField-clear', undefined, 'clear'), 0, {
              "href": "#clear",
              "onClick": this.doUnset
            })]
          })
        }), createComponentVNode(2, Calendar, {
          "className": "InfernoFormlib-DateFieldCalendar",
          "showMonth": this.state.showMonth,
          "value": this.state.value,
          "placement": "bottom",
          "isOpen": this.state.popoverOpen,
          "target": escapeIdSelector(inputId),
          "onNavigate": this.doUpdateShowMonth,
          "onShowSelected": this.doShowSelected,
          "onSelect": this.doUpdateValue
        })]
      }, null, function (el) {
        return _this4._el = el;
      });
    }
  }]);

  return InputWidget;
}(Component);

export default InputWidget;
new Adapter({
  implements: IInputFieldWidget,
  adapts: interfaces.IDateField,
  Component: InputWidget
});
//# sourceMappingURL=DateField.js.map