'use strict';
/*

    To use this input widget adapter you need to register it with your
    adapter registry.

*/

import _objectSpread from "@babel/runtime/helpers/objectSpread";
import _classCallCheck from "@babel/runtime/helpers/classCallCheck";
import _createClass from "@babel/runtime/helpers/createClass";
import _possibleConstructorReturn from "@babel/runtime/helpers/possibleConstructorReturn";
import _getPrototypeOf from "@babel/runtime/helpers/getPrototypeOf";
import _inherits from "@babel/runtime/helpers/inherits";
import _assertThisInitialized from "@babel/runtime/helpers/assertThisInitialized";
import { createComponentVNode, normalizeProps } from "inferno";
import { Component } from 'inferno';
import { Adapter } from 'component-registry';
import { interfaces } from 'isomorphic-schema';
import { IInputFieldWidget } from '../interfaces';
import { renderString } from './common';
import { generateId } from './utils';
import { Input } from 'inferno-bootstrap'; // Placeholder

var InputWidget =
/*#__PURE__*/
function (_Component) {
  _inherits(InputWidget, _Component);

  function InputWidget(props) {
    var _this;

    _classCallCheck(this, InputWidget);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(InputWidget).call(this, props));
    _this.state = {
      value: props.value
    };
    _this.didGetInput = _this.didGetInput.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.didGetChange = _this.didGetChange.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    return _this;
  }

  _createClass(InputWidget, [{
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      this.setState({
        value: nextProps.value
      });
    }
  }, {
    key: "didGetInput",
    value: function didGetInput(e) {
      var field = this.props.adapter.context;
      var newVal = field.fromString(e.target.value);
      this.setState({
        value: newVal
      }); // Trigger change so FormRow can update character counter

      this.props.onChange(this.props.propName, newVal);
    }
  }, {
    key: "didGetChange",
    value: function didGetChange(e) {
      this.props.onChange(this.props.propName, this.state.value);
    }
  }, {
    key: "render",
    value: function render(_ref) {
      var inputName = _ref.inputName,
          namespace = _ref.namespace,
          options = _ref.options,
          id = _ref.id,
          doesNotRenderLabel = _ref.doesNotRenderLabel;
      var field = this.props.adapter.context;
      var isValid = this.props.validationError || this.props.invariantError ? false : undefined;
      var ariaLabels = {
        'aria-invalid': isValid !== undefined,
        'aria-labelledby': doesNotRenderLabel ? undefined : id,
        'aria-label': doesNotRenderLabel ? renderString(field.label || 'inferno-formlib--InputField', options && options.lang, 'Input Field') : undefined,
        'aria-required': field._isRequired ? field._isRequired : undefined
      };
      return normalizeProps(createComponentVNode(2, Input, _objectSpread({
        "id": generateId(namespace, '__Field'),
        "name": inputName,
        "valid": isValid,
        "placeholder": renderString(field.placeholder, options && options.lang),
        "readOnly": field.readOnly,
        "value": field.toFormattedString(this.state.value)
      }, ariaLabels, {
        "onInput": this.didGetInput,
        "onChange": this.didGetChange
      })));
    }
  }]);

  return InputWidget;
}(Component);

export default InputWidget;
new Adapter({
  implements: IInputFieldWidget,
  adapts: interfaces.ITextField,
  Component: InputWidget
});
new Adapter({
  implements: IInputFieldWidget,
  adapts: interfaces.IIntegerField,
  Component: InputWidget
});

var DecimalWidget =
/*#__PURE__*/
function (_InputWidget) {
  _inherits(DecimalWidget, _InputWidget);

  function DecimalWidget() {
    _classCallCheck(this, DecimalWidget);

    return _possibleConstructorReturn(this, _getPrototypeOf(DecimalWidget).apply(this, arguments));
  }

  _createClass(DecimalWidget, [{
    key: "didGetInput",
    value: function didGetInput(e) {
      var field = this.props.adapter.context;
      var inp = e.target.value; // Preserve content in input box if ending with decimal and not has any previous decimal

      if (typeof inp === 'string' && inp[inp.length - 1] === '.' && inp.match(/\./g).length === 1) {
        return this.setState({
          value: inp
        });
      }

      var newVal = field.fromString(e.target.value);
      this.setState({
        value: newVal
      }); // Trigger change so FormRow can update character counter

      this.props.onChange(this.props.propName, newVal);
    }
  }, {
    key: "render",
    value: function render(_ref2) {
      var inputName = _ref2.inputName,
          namespace = _ref2.namespace,
          options = _ref2.options,
          id = _ref2.id,
          doesNotRenderLabel = _ref2.doesNotRenderLabel;
      var field = this.props.adapter.context;
      var isValid = this.props.validationError || this.props.invariantError ? false : undefined;
      var value = this.state.value === undefined || this.state.value === null ? '' : field.toFormattedString(this.state.value);
      var ariaLabels = {
        'aria-invalid': isValid !== undefined,
        'aria-labelledby': doesNotRenderLabel ? undefined : id,
        'aria-label': doesNotRenderLabel ? renderString(field.label || 'inferno-formlib--InputField', options && options.lang, 'Input Field') : undefined,
        'aria-required': field._isRequired ? field._isRequired : undefined
      };
      return normalizeProps(createComponentVNode(2, Input, _objectSpread({
        "id": generateId(namespace, '__Field'),
        "name": inputName,
        "valid": isValid,
        "placeholder": renderString(field.placeholder, options && options.lang),
        "readOnly": field.readOnly,
        "value": value
      }, ariaLabels, {
        "onInput": this.didGetInput,
        "onChange": this.didGetChange
      })));
    }
  }]);

  return DecimalWidget;
}(InputWidget);

new Adapter({
  implements: IInputFieldWidget,
  adapts: interfaces.IDecimalField,
  Component: DecimalWidget
});
//# sourceMappingURL=InputField.js.map