'use strict';
/*

    To use this input widget adapter you need to register it with your
    adapter registry.

*/

import _typeof from "@babel/runtime/helpers/typeof";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _classCallCheck from "@babel/runtime/helpers/classCallCheck";
import _createClass from "@babel/runtime/helpers/createClass";
import _possibleConstructorReturn from "@babel/runtime/helpers/possibleConstructorReturn";
import _getPrototypeOf from "@babel/runtime/helpers/getPrototypeOf";
import _inherits from "@babel/runtime/helpers/inherits";
import _assertThisInitialized from "@babel/runtime/helpers/assertThisInitialized";
import { createVNode, createComponentVNode } from "inferno";
import { Adapter } from 'component-registry';
import { Component } from 'inferno';
import { findDOMNode } from 'inferno-extras';
import { safeGet } from 'safe-utils';
import { renderString } from './common';
import { i18n } from 'isomorphic-schema';
import { interfaces } from 'isomorphic-schema';
import { IInputFieldWidget } from '../interfaces';
import { generateId } from './utils';
import { handleDragStart, handleDragOver, handleDragEnter, handleDragLeave, handleDragEnd, handleDrop } from '../draggable';
import { Placeholder, renderRows } from './ListFieldHelpers';

var ListFieldWidget =
/*#__PURE__*/
function (_Component) {
  _inherits(ListFieldWidget, _Component);

  function ListFieldWidget(props) {
    var _this;

    _classCallCheck(this, ListFieldWidget);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(ListFieldWidget).call(this, props)); // Perhaps we should do a capability test and throw an error if not array like

    if (props.value && !(props.value instanceof Array)) throw new Error('Value passed to ListFieldWidget is not an array');
    var keys = {};
    _this.keysNext = 0;
    _this.keys = []; // Initialise keys for passed array if any
    // Making this test less strict to support Mobx Arrays:

    if (props.value instanceof Array) {
      for (var i = 0; i < props.value.length; i++) {
        _this.keys.push({
          key: _this.keysNext,
          justAdded: false
        });

        _this.keysNext++;
      }
    }

    _this.isMounted = false;
    _this.didInput = _this.didInput.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.didUpdate = _this.didUpdate.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.doAddRow = _this.doAddRow.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.doDeleteRow = _this.doDeleteRow.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.didDrop = _this.didDrop.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.handleDragStart = handleDragStart.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.handleDragOver = handleDragOver.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.handleDragEnter = handleDragEnter.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.handleDragLeave = handleDragLeave.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.handleDragEnd = handleDragEnd.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.handleDrop = handleDrop.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    return _this;
  }

  _createClass(ListFieldWidget, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      var domEl = findDOMNode(this);
      domEl.addEventListener('dragstart', this.handleDragStart, false);
      domEl.addEventListener('dragover', this.handleDragOver, false);
      domEl.addEventListener('dragenter', this.handleDragEnter, false);
      domEl.addEventListener('dragleave', this.handleDragLeave, false);
      domEl.addEventListener('dragend', this.handleDragEnd, false);
      domEl.addEventListener('drop', this.handleDrop, false);
      this.listenersAdded = true; // Because this is a container we set isMounted here instead of getting it from parent

      this.isMounted = true;
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      var domEl = findDOMNode(this);
      domEl.removeEventListener('dragstart', this.handleDragStart, false);
      domEl.removeEventListener('dragover', this.handleDragOver, false);
      domEl.removeEventListener('dragenter', this.handleDragEnter, false);
      domEl.removeEventListener('dragleave', this.handleDragLeave, false);
      domEl.removeEventListener('dragend', this.handleDragEnd, false);
      domEl.removeEventListener('drop', this.handleDrop, false);
    }
  }, {
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(nextProps) {
      // Making this test less strict to support Mobx Arrays:
      if (!(nextProps.value instanceof Array) || nextProps.value.length < this.keys.length) {
        // We got undefined or fewer values than previously, need to shorten the keys array
        this.keys.splice(safeGet(function () {
          return nextProps.value.length;
        }, 0));
        return;
      } // New array is larger so need to add som keys


      for (var i = this.keys.length; i < nextProps.value.length; i++) {
        this.keys.push({
          key: this.keysNext,
          justAdded: true
        });
        this.keysNext++;
      }
    }
  }, {
    key: "didInput",
    value: function didInput(propName, data) {
      var newValue = _toConsumableArray(this.props.value);

      newValue[propName] = data;
      this.props.onInput && this.props.onInput(this.props.propName, newValue);
    }
  }, {
    key: "didUpdate",
    value: function didUpdate(propName, data) {
      var newValue = _toConsumableArray(this.props.value);

      newValue[propName] = data;
      this.props.onChange(this.props.propName, newValue);
    }
  }, {
    key: "doAddRow",
    value: function doAddRow(e) {
      e.preventDefault(); // Adding this because in some cases the event will keep on calling this event handler.
      // Could be due to propagation.

      e.stopPropagation();
      var value = this.props.value || [];

      var newValue = _toConsumableArray(value);

      newValue.push(undefined);
      this.props.onChange(this.props.propName, newValue);
    }
  }, {
    key: "doDeleteRow",
    value: function doDeleteRow(index) {
      var value = this.props.value;
      var newValue = value.filter(function (item, i) {
        return i !== index;
      });
      this.props.onChange(this.props.propName, newValue);
    } // TODO: This should probably be removed

  }, {
    key: "didDrop",
    value: function didDrop(source, targetIndex) {
      var newValue = _toConsumableArray(this.props.value);

      var sourceKey;
      var sourceObj;

      if (_typeof(source) === 'object') {
        sourceObj = source;
        sourceKey = {
          key: this.keysNext,
          justAdded: true
        };
        this.keysNext++;
      } else {
        var sourceIndex = source;
        sourceObj = newValue.splice(sourceIndex, 1)[0];
        sourceKey = this.keys.splice(sourceIndex, 1)[0];

        if (sourceIndex < targetIndex) {
          targetIndex--;
        }
      }

      newValue.splice(targetIndex, 0, sourceObj);
      this.keys.splice(targetIndex, 0, sourceKey);
      this.props.onChange(this.props.propName, newValue);
    }
  }, {
    key: "renderAddButton",
    value: function renderAddButton(_ref) {
      var adapter = _ref.adapter,
          value = _ref.value,
          options = _ref.options;
      var field = adapter.context;
      var nrofItems = Array.isArray(value) && value.length || 0; // Respect maxLength

      if (field.maxLength <= nrofItems) return null;
      return createVNode(1, "div", "InfernoFormlib-ListFieldActionBar", createVNode(64, "input", "btn btn-primary", null, 1, {
        "type": "button",
        "value": renderString(i18n('inferno-formlib--ListField_add', 'Add'), options && options.lang, 'Add'),
        "onClick": this.doAddRow
      }), 2);
    }
  }, {
    key: "render",
    value: function render(_ref2) {
      var adapter = _ref2.adapter,
          value = _ref2.value,
          namespace = _ref2.namespace,
          options = _ref2.options;
      var field = adapter.context;
      var emptyArray = !Array.isArray(value) || value.length === 0;
      return createVNode(1, "div", "InfernoFormlib-ListField InfernoFormlib-DragContainer", [emptyArray && field.placeholder && createComponentVNode(2, ListFieldRow, {
        "isFirstMount": !this.props.formIsMounted,
        children: createComponentVNode(2, Placeholder, {
          "text": renderString(field.placeholder)
        })
      }, "placeholder"), renderRows({
        lang: options.lang,
        field: field,
        value: value || [],
        namespace: namespace || [],
        inputName: this.props.inputName,
        itemKeys: this.keys,
        validationErrors: this.props.validationError,
        customWidgets: this.props.customWidgets,
        onInput: this.didInput,
        onChange: this.didUpdate,
        onDelete: this.doDeleteRow,
        isMounted: this.isMounted
      }), this.renderAddButton({
        adapter: adapter,
        value: value,
        options: options
      })], 0, {
        "id": generateId(namespace, '__Field')
      });
    }
  }]);

  return ListFieldWidget;
}(Component);

export { ListFieldWidget as default };
new Adapter({
  implements: IInputFieldWidget,
  adapts: interfaces.IListField,
  Component: ListFieldWidget
});
//# sourceMappingURL=ListField.js.map